#ifndef __SYSTEM_CONFIG_H_
#define __SYSTEM_CONFIG_H_


#define FPGA_DEVICE_FAMILY    "MachXO2"
#define PLATFORM_NAME         "platform"
#define USE_PLL               (0)
#define CPU_FREQUENCY         (25000000)


/* FOUND 1 CPU UNIT(S) */

/*
 * CPU Instance LM32 component configuration
 */
#define CPU_NAME "LM32"
#define CPU_EBA (0x02000000)
#define CPU_DIVIDE_ENABLED (0)
#define CPU_SIGN_EXTEND_ENABLED (0)
#define CPU_MULTIPLIER_ENABLED (1)
#define CPU_SHIFT_ENABLED (1)
#define CPU_DEBUG_ENABLED (1)
#define CPU_HW_BREAKPOINTS_ENABLED (0)
#define CPU_NUM_HW_BREAKPOINTS (0)
#define CPU_NUM_WATCHPOINTS (0)
#define CPU_ICACHE_ENABLED (0)
#define CPU_ICACHE_SETS (512)
#define CPU_ICACHE_ASSOC (1)
#define CPU_ICACHE_BYTES_PER_LINE (16)
#define CPU_DCACHE_ENABLED (0)
#define CPU_DCACHE_SETS (512)
#define CPU_DCACHE_ASSOC (1)
#define CPU_DCACHE_BYTES_PER_LINE (16)
#define CPU_DEBA (0x00000000)
#define CPU_CHARIO_IN        (1)
#define CPU_CHARIO_OUT       (1)
#define CPU_CHARIO_TYPE      "JTAG UART"

/*
 * gpio component configuration
 */
#define GPIO_NAME  "gpio"
#define GPIO_BASE_ADDRESS  (0x84000000)
#define GPIO_SIZE  (16)
#define GPIO_CHARIO_IN        (0)
#define GPIO_CHARIO_OUT       (0)
#define GPIO_WB_DAT_WIDTH  (32)
#define GPIO_WB_ADR_WIDTH  (4)
#define GPIO_ADDRESS_LOCK  (1)
#define GPIO_DISABLE  (0)
#define GPIO_OUTPUT_PORTS_ONLY  (1)
#define GPIO_INPUT_PORTS_ONLY  (0)
#define GPIO_TRISTATE_PORTS  (0)
#define GPIO_BOTH_INPUT_AND_OUTPUT  (0)
#define GPIO_DATA_WIDTH  (4)
#define GPIO_INPUT_WIDTH  (1)
#define GPIO_OUTPUT_WIDTH  (1)
#define GPIO_IRQ_MODE  (0)
#define GPIO_LEVEL  (0)
#define GPIO_EDGE  (0)
#define GPIO_EITHER_EDGE_IRQ  (0)
#define GPIO_POSE_EDGE_IRQ  (0)
#define GPIO_NEGE_EDGE_IRQ  (0)

/*
 * ebr component configuration
 */
#define EBR_NAME  "ebr"
#define EBR_BASE_ADDRESS  (0x02000000)
#define EBR_SIZE  (22528)
#define EBR_IS_READABLE   (1)
#define EBR_IS_WRITABLE   (1)
#define EBR_ADDRESS_LOCK  (1)
#define EBR_DISABLE  (0)
#define EBR_WB_DAT_WIDTH  (32)
#define EBR_INIT_FILE_NAME  "C:/work/project/artekit/mico32_ver0/software/init_mem.mem"
#define EBR_INIT_FILE_FORMAT  "hex"

/*
 * slave_passthru component configuration
 */
#define SLAVE_PASSTHRU_NAME  "slave_passthru"
#define SLAVE_PASSTHRU_BASE_ADDRESS  (0x80000000)
#define SLAVE_PASSTHRU_SIZE  (67108864)
#define SLAVE_PASSTHRU_IRQ (0)
#define SLAVE_PASSTHRU_CHARIO_IN        (0)
#define SLAVE_PASSTHRU_CHARIO_OUT       (0)
#define SLAVE_PASSTHRU_ADDRESS_LOCK  (1)
#define SLAVE_PASSTHRU_DISABLE  (0)
#define SLAVE_PASSTHRU_WB_DAT_WIDTH  (32)
#define SLAVE_PASSTHRU_WB_SEL_WIDTH  (4)
#define SLAVE_PASSTHRU_WB_ADR_WIDTH  (32)

/*
 * pagesel component configuration
 */
#define PAGESEL_NAME  "pagesel"
#define PAGESEL_BASE_ADDRESS  (0x88000000)
#define PAGESEL_SIZE  (16)
#define PAGESEL_CHARIO_IN        (0)
#define PAGESEL_CHARIO_OUT       (0)
#define PAGESEL_WB_DAT_WIDTH  (32)
#define PAGESEL_WB_ADR_WIDTH  (4)
#define PAGESEL_ADDRESS_LOCK  (1)
#define PAGESEL_DISABLE  (0)
#define PAGESEL_OUTPUT_PORTS_ONLY  (1)
#define PAGESEL_INPUT_PORTS_ONLY  (0)
#define PAGESEL_TRISTATE_PORTS  (0)
#define PAGESEL_BOTH_INPUT_AND_OUTPUT  (0)
#define PAGESEL_DATA_WIDTH  (8)
#define PAGESEL_INPUT_WIDTH  (1)
#define PAGESEL_OUTPUT_WIDTH  (1)
#define PAGESEL_IRQ_MODE  (0)
#define PAGESEL_LEVEL  (0)
#define PAGESEL_EDGE  (0)
#define PAGESEL_EITHER_EDGE_IRQ  (0)
#define PAGESEL_POSE_EDGE_IRQ  (0)
#define PAGESEL_NEGE_EDGE_IRQ  (0)

/*
 * SPI_REG component configuration
 */
#define SPI_REG_NAME  "SPI_REG"
#define SPI_REG_BASE_ADDRESS  (0x8C000000)
#define SPI_REG_SIZE  (256)
#define SPI_REG_IRQ (1)
#define SPI_REG_CHARIO_IN        (0)
#define SPI_REG_CHARIO_OUT       (0)
#define SPI_REG_ADDRESS_LOCK  (1)
#define SPI_REG_DISABLE  (0)
#define SPI_REG_WB_DAT_WIDTH  (32)
#define SPI_REG_WB_SEL_WIDTH  (4)
#define SPI_REG_WB_ADR_WIDTH  (32)

/*
 * tasti component configuration
 */
#define TASTI_NAME  "tasti"
#define TASTI_BASE_ADDRESS  (0x90000000)
#define TASTI_SIZE  (16)
#define TASTI_CHARIO_IN        (0)
#define TASTI_CHARIO_OUT       (0)
#define TASTI_WB_DAT_WIDTH  (32)
#define TASTI_WB_ADR_WIDTH  (4)
#define TASTI_ADDRESS_LOCK  (1)
#define TASTI_DISABLE  (0)
#define TASTI_OUTPUT_PORTS_ONLY  (0)
#define TASTI_INPUT_PORTS_ONLY  (1)
#define TASTI_TRISTATE_PORTS  (0)
#define TASTI_BOTH_INPUT_AND_OUTPUT  (0)
#define TASTI_DATA_WIDTH  (8)
#define TASTI_INPUT_WIDTH  (1)
#define TASTI_OUTPUT_WIDTH  (1)
#define TASTI_IRQ_MODE  (0)
#define TASTI_LEVEL  (0)
#define TASTI_EDGE  (0)
#define TASTI_EITHER_EDGE_IRQ  (0)
#define TASTI_POSE_EDGE_IRQ  (0)
#define TASTI_NEGE_EDGE_IRQ  (0)


#endif /* __SYSTEM_CONFIG_H_ */
