
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity SPI_top is
   port(
      -- system interface
		rst        : in std_logic;
		clk : in std_logic;
     
      u_spi_csn           : in std_logic;
      u_spi_clk           : in std_logic;
      u_spi_d             : in std_logic;
      u_spi_q             : out std_logic;

	    WB_RST : IN std_logic;
		WB_CLK : IN std_logic;
		WB_CTI_I : IN std_logic_vector(2 downto 0);
		WB_BTE_I : IN std_logic_vector(1 downto 0);
		WB_ADR_I : IN std_logic_vector(31 downto 0);
		WB_DAT_I : IN std_logic_vector(31 downto 0);
		WB_SEL_I : IN std_logic_vector(3 downto 0);
		WB_WE_I : IN std_logic;
		WB_STB_I : IN std_logic;
		WB_CYC_I : IN std_logic;
		WB_ACK_O : OUT std_logic;
		WB_DAT_O : OUT std_logic_vector(31 downto 0);
		WB_ERR_O : OUT std_logic;
		WB_RTY_O : OUT std_logic;
		RDY : OUT std_logic

   );
end SPI_top;
-----------------------------------------------------------------------
architecture rtl_SPI_top of SPI_top is
component memoria
    port (WrAddress: in  std_logic_vector(3 downto 0); 
        Data: in  std_logic_vector(15 downto 0); WrClock: in  std_logic; 
        WE: in  std_logic; WrClockEn: in  std_logic; 
        RdAddress: in  std_logic_vector(3 downto 0); 
        Q: out  std_logic_vector(15 downto 0));
end component;
constant clk_rising : std_logic := '1';
signal input_SR : std_logic_vector (15 downto 0);
signal u_spi_clk_smp1 : std_logic;  
signal u_spi_clk_smp2 : std_logic; 
signal u_spi_csn_smp1 : std_logic;  
signal u_spi_csn_smp2 : std_logic;  
signal u_spi_csn_smp3 : std_logic; 
signal WrAddress 		: std_logic_vector(3 downto 0);
signal Data				: std_logic_vector(15 downto 0);
signal WE_int : std_logic;  
signal WE : std_logic; 

signal WE_int2 : std_logic; 
signal WE_int_smp : std_logic; 

signal Q				: std_logic_vector(15 downto 0);
signal RdAddress        :std_logic_vector(3 downto 0);
begin


process(rst, WB_CLK)
begin
if rst = '0' then
	WB_ACK_O <= '0';
elsif WB_CLK'event and WB_CLK='1' then
	WB_ACK_O <= WB_STB_I;
end if;
end process;
---------- input sr
process(rst, WB_CLK)
begin
if rst = '0' then
	input_SR <= "0000000000000000";
	WE_int <= '0';
	WE_int2 <= '0';
	WE_int_smp <= '0';
	RDY <= '0';
elsif WB_CLK'event and WB_CLK='1' then
	u_spi_clk_smp1 <= u_spi_clk;
	u_spi_clk_smp2 <= u_spi_clk_smp1;
	u_spi_csn_smp1 <= u_spi_csn;
	u_spi_csn_smp2 <= u_spi_csn_smp1;
	u_spi_csn_smp3 <= u_spi_csn_smp2;
	WE_int <= '0';
	WE <= WE_int or WE_int2;
	WE_int_smp <= WE_int ;
	
	if u_spi_clk_smp2 = '1' and u_spi_clk_smp1 = '0' and u_spi_csn = '0' then -- discesa (attenzione inverso al cliente), CS attivo basso
		input_SR(15 downto 0) <= input_SR(14 downto 0) & u_spi_d;
	end if;
	
	if u_spi_csn_smp3 = '0' and u_spi_csn_smp2 = '1'  then -- salita CS
		if 		input_SR(15 downto 8) = "00000000" then
			WrAddress <= input_SR(3 downto 0) ;
		elsif	input_SR(15 downto 8) = "00000001" then
			Data(15 downto 8) <= input_SR(7 downto 0);
		elsif	input_SR(15 downto 8) = "00000010" then
			Data(7 downto 0) <= input_SR(7 downto 0);
				WE_int <= '1';
		end if;
	end if;
	
	if WrAddress = "1111" and WE_int_smp = '1' then --and Data = "1111111111111111" then
		RDY <= '1';
	elsif WB_STB_I = '1' and WB_CYC_I ='1' and WB_WE_I='1' then
		RDY <= '0';
		WrAddress <= "1111" ;
		Data <= (others => '0');
		WE_int2 <= '1';
	end if;
	

	
end if;
end process;

inst_memoria : memoria
    port map (
	WrAddress(3 downto 0)=>WrAddress, 
	Data(15 downto 0)=>Data, 
	WrClock=>WB_CLK, 
    WE=>WE, 
	WrClockEn=>'1', 
	RdAddress(3 downto 0)=>RdAddress, 
	Q(15 downto 0)=>Q);


WB_DAT_O <= Q & Q;

RdAddress(3 downto 0)  <= WB_ADR_I(4 downto 1);


end rtl_SPI_top;
-----------------------------------------------------------------------