
--
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY testbench IS
END testbench;

ARCHITECTURE behavior OF testbench IS 

	COMPONENT spi_top_RW
	PORT(
		rst : IN std_logic;
		clk : IN std_logic;
		u_spi_csn : IN std_logic;
		u_spi_clk : IN std_logic;
		u_spi_d : IN std_logic;
		WB_RST : IN std_logic;
		WB_CLK : IN std_logic;
		WB_CTI_I : IN std_logic_vector(2 downto 0);
		WB_BTE_I : IN std_logic_vector(1 downto 0);
		WB_ADR_I : IN std_logic_vector(31 downto 0);
		WB_DAT_I : IN std_logic_vector(31 downto 0);
		WB_SEL_I : IN std_logic_vector(3 downto 0);
		WB_WE_I : IN std_logic;
		WB_STB_I : IN std_logic;
		WB_CYC_I : IN std_logic;          
		u_spi_q : OUT std_logic;
		WB_ACK_O : OUT std_logic;
		WB_DAT_O : OUT std_logic_vector(31 downto 0);
		WB_ERR_O : OUT std_logic;
		WB_RTY_O : OUT std_logic;
				RDY : OUT std_logic
		);
	END COMPONENT;

	SIGNAL rst :  std_logic;
	SIGNAL clk :  std_logic;
	SIGNAL u_spi_csn :  std_logic;
	SIGNAL u_spi_clk :  std_logic;
	SIGNAL u_spi_d :  std_logic;
	SIGNAL u_spi_q :  std_logic;
	SIGNAL WB_RST :  std_logic;
	SIGNAL WB_CLK :  std_logic;
	SIGNAL WB_CTI_I :  std_logic_vector(2 downto 0);
	SIGNAL WB_BTE_I :  std_logic_vector(1 downto 0);
	SIGNAL WB_ADR_I :  std_logic_vector(31 downto 0);
	SIGNAL WB_DAT_I :  std_logic_vector(31 downto 0);
	SIGNAL WB_SEL_I :  std_logic_vector(3 downto 0);
	SIGNAL WB_WE_I :  std_logic;
	SIGNAL WB_STB_I :  std_logic;
	SIGNAL WB_CYC_I :  std_logic;
	SIGNAL WB_ACK_O :  std_logic;
	SIGNAL WB_DAT_O :  std_logic_vector(31 downto 0);
	SIGNAL WB_ERR_O :  std_logic;
	SIGNAL WB_RTY_O :  std_logic;
	signal		RDY : std_logic;
signal dataR :  std_logic_vector(15 downto 0);
	constant spi_semiperiod : time := 7000 ns;
	constant preamble_address  : std_logic_vector (7 downto 0) := X"00";
	constant preamble_dataHI  : std_logic_vector (7 downto 0) := X"01";
	constant preamble_dataLOW  : std_logic_vector (7 downto 0) := X"02";
	
	constant preamble_addressR  : std_logic_vector (7 downto 0) := X"80";
	constant preamble_dataHIR  : std_logic_vector (7 downto 0) := X"81";
	constant preamble_dataLOWR  : std_logic_vector (7 downto 0) := X"82";
	
	constant postamble : std_logic_vector (7 downto 0) := X"00";
procedure spi_write (
		address : in std_logic_vector(7 downto 0);
		data : in std_logic_vector(15 downto 0); 
		signal u_spi_csn : out std_logic;
		signal u_spi_clk : out std_logic;
		signal u_spi_d : out std_logic;
		signal u_spi_q : in std_logic) is
	
begin

u_spi_csn 	<= '1';
u_spi_clk 	<= '1';
u_spi_d 	<= '1';
wait for 30 us;
wait for  spi_semiperiod;

-- preambolo address
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= preamble_address(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--address
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= address(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

-- preambolo dataHI
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= preamble_dataHI(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--data
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= data(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

-- preambolo dataLOW
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= preamble_dataLOW(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--data
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= data(i+8);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

u_spi_csn 	<= '1';
u_spi_clk 	<= '1';
u_spi_d 	<= '1';
wait for 100 ns;
end spi_write;	

---------------------------------------------------------------------------------------------------------------------------
procedure spi_read (
		address : in std_logic_vector(7 downto 0);
		signal data : out std_logic_vector(15 downto 0); 
		signal u_spi_csn : out std_logic;
		signal u_spi_clk : out std_logic;
		signal u_spi_d : out std_logic ;
		signal u_spi_q : in std_logic) is
	
begin

u_spi_csn 	<= '1';
u_spi_clk 	<= '1';
u_spi_d 	<= '1';
wait for 30 us;
wait for  spi_semiperiod;

-- preambolo address
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= preamble_addressR(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--address
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= address(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

-- preambolo dataHI
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= '1';--preamble_dataHIR(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--data
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= '0';
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

-- preambolo dataLOW
u_spi_csn 	<= '0';
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= '1'; --preamble_dataLOWR(i);
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
--data
for i in 7 downto 0 loop
	u_spi_clk 	<= '1';
	u_spi_d <= '0';
	wait for  spi_semiperiod;
	u_spi_clk 	<= '0';
	wait for  spi_semiperiod;
end loop;
u_spi_csn 	<= '1';
wait for 30 us;

u_spi_csn 	<= '1';
u_spi_clk 	<= '1';
u_spi_d 	<= '1';
wait for 100 ns;
end spi_read;	
---------------------------------------------------------------------------------------------------------------------------
	
procedure s_read (
		address : in std_logic_vector(31 downto 0);
		data : in std_logic_vector(31 downto 0); 
		sel   : in std_logic_vector(3 downto 0); 
	signal ADR_I  : out std_logic_vector(31 downto 0);
	signal DAT_I  : out std_logic_vector(31 downto 0);
	signal WE_I   : out std_logic;
	signal STB_I  : out std_logic;
	signal CYC_I  : out std_logic;
	signal CTI_I  : out std_logic_vector(2 downto 0);
	signal SEL_I  : out std_logic_vector(3 downto 0);
	signal BTE_I  : out std_logic_vector(1 downto 0);
	signal DAT_O : in std_logic_vector(31 downto 0);
	signal ACK_O : in std_logic;	
	signal CLK_I : in std_logic;
	signal RST_I : in std_logic ) is
	
begin
ADR_I <= (others => '0');
DAT_I <= (others => '0');
WE_I  <= '0';
STB_I <= '0';
CYC_I <= '0';
CTI_I <= (others => '0');
SEL_I <= (others => '0');
BTE_I <= (others => '0');
wait until (CLK_I'event and CLK_I='1');
ADR_I <= address;
WE_I  <= '0';
STB_I <= '1';
CYC_I <= '1';
CTI_I <= "000";
SEL_I <= sel;
BTE_I <= "00";
wait until (ACK_O'event and ACK_O='1');
wait until (CLK_I'event and CLK_I='1');
ADR_I <= (others => '0');
DAT_I <= (others => '0');
WE_I  <= '0';
STB_I <= '0';
CYC_I <= '0';
CTI_I <= (others => '0');
SEL_I <= (others => '0');
BTE_I <= (others => '0');
wait until (CLK_I'event and CLK_I='1');
end s_read;


procedure s_write (
		address : in std_logic_vector(31 downto 0);
		data : in std_logic_vector(31 downto 0); 
		sel   : in std_logic_vector(3 downto 0); 
	signal ADR_I  : out std_logic_vector(31 downto 0);
	signal DAT_I  : out std_logic_vector(31 downto 0);
	signal WE_I   : out std_logic;
	signal STB_I  : out std_logic;
	signal CYC_I  : out std_logic;
	signal CTI_I  : out std_logic_vector(2 downto 0);
	signal SEL_I  : out std_logic_vector(3 downto 0);
	signal BTE_I  : out std_logic_vector(1 downto 0);
	signal DAT_O : in std_logic_vector(31 downto 0);
	signal ACK_O : in std_logic;	
	signal CLK_I : in std_logic;
	signal RST_I : in std_logic ) is
	
begin
ADR_I <= (others => '0');
DAT_I <= (others => '0');
WE_I  <= '0';
STB_I <= '0';
CYC_I <= '0';
CTI_I <= (others => '0');
SEL_I <= (others => '0');
BTE_I <= (others => '0');
wait until (CLK_I'event and CLK_I='1');
ADR_I <= address;
WE_I  <= '1';
STB_I <= '1';
CYC_I <= '1';
CTI_I <= "000";
SEL_I <= sel;
BTE_I <= "00";
wait until (ACK_O'event and ACK_O='1');
wait until (CLK_I'event and CLK_I='1');
ADR_I <= (others => '0');
DAT_I <= (others => '0');
WE_I  <= '0';
STB_I <= '0';
CYC_I <= '0';
CTI_I <= (others => '0');
SEL_I <= (others => '0');
BTE_I <= (others => '0');
wait until (CLK_I'event and CLK_I='1');
end s_write;

	
BEGIN

-- Please check and add your generic clause manually
	uut: spi_top_RW PORT MAP(
		rst => rst,
		clk => clk,
		u_spi_csn => u_spi_csn,
		u_spi_clk => u_spi_clk,
		u_spi_d => u_spi_d,
		u_spi_q => u_spi_q,
		WB_RST => WB_RST,
		WB_CLK => WB_CLK,
		WB_CTI_I => WB_CTI_I,
		WB_BTE_I => WB_BTE_I,
		WB_ADR_I => WB_ADR_I,
		WB_DAT_I => WB_DAT_I,
		WB_SEL_I => WB_SEL_I,
		WB_WE_I => WB_WE_I,
		WB_STB_I => WB_STB_I,
		WB_CYC_I => WB_CYC_I,
		WB_ACK_O => WB_ACK_O,
		WB_DAT_O => WB_DAT_O,
		WB_ERR_O => WB_ERR_O,
		WB_RTY_O => WB_RTY_O,
		RDY => RDY
	);

process
begin
WB_CLK <= '0' ;
wait for 20 ns;
WB_CLK <= '1' ;
wait for 20 ns;
end process;

process
begin
WB_RST <= '0';
wait for 100 ns;
WB_RST <= '1';
wait;
end process;

rst <= WB_RST;
clk <= WB_CLK;




-- *** Test Bench - User Defined Section ***
   tb : PROCESS
   BEGIN	
		WB_ADR_I <= (others => '0');
	   wait for 200 ns;
	   spi_write (X"01", X"3333",u_spi_csn , u_spi_clk , u_spi_d , u_spi_q ) ;	
	   wait for 200 ns;
	   spi_write (X"0F", X"FFFF",u_spi_csn , u_spi_clk , u_spi_d , u_spi_q) ;
       wait for 50 us;
	   




	   
	   wait for 50 us;
	   spi_read (X"00", dataR,u_spi_csn , u_spi_clk , u_spi_d , u_spi_q) ;	
	   wait for 200 ns;
	   spi_read (X"01", dataR,u_spi_csn , u_spi_clk , u_spi_d , u_spi_q) ;	
	   wait for 200 ns;
	   spi_read (X"02", dataR,u_spi_csn , u_spi_clk , u_spi_d , u_spi_q) ;	
	   wait;
	   wait for 100000 ns;
	   s_read (X"00000000", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   s_read (X"00000002", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   s_read (X"00000004", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   s_read (X"00000006", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   
	   --s_WRITE (X"0000000F", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   -- wait for 200 ns;
	   s_read (X"00000000", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   wait for 200 ns;
	   s_read (X"00000002", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   wait for 200 ns;
	   spi_write (X"02", X"1111",u_spi_csn , u_spi_clk , u_spi_d , u_spi_q ) ;
	   s_read (X"00000004", X"00000000", X"F", WB_ADR_I, WB_DAT_I, WB_WE_I, WB_STB_I, WB_CYC_I, WB_CTI_I, WB_SEL_I, WB_BTE_I, WB_DAT_O, WB_ACK_O, WB_CLK, WB_RST );
	   wait for 200 ns;
      wait; -- will wait forever
   END PROCESS;
-- *** End Test Bench - User Defined Section ***


END;
