-- ====================================================================
-- micro_emulator.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver x.x - date xx gen 2007
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity micro_emulator is
   port(
      -- system interface
      sys_rst         : in std_logic;
      tft_clk         : in std_logic;
      demo_start      : in std_logic;
      -- micro_ctrl_if interface
      demo_sel        : out std_logic; -- OPEN
      tft_surfer_busy : out std_logic; -- TEST_POINT
      -- img_mover_viewer interface
      img_start_move  : out std_logic; -- attivo alto un solo tft_clk
      img_view        : out std_logic; -- attivo alto un solo tft_clk
      img_nr          : out std_logic_vector(3 downto 0);
      img_end_move    : in std_logic
   );
end micro_emulator;
-----------------------------------------------------------------------
architecture rtl_micro_emulator of micro_emulator is

   type demo_fsm_state is (IDLE, START_IMG_VIEW_ST, WAIT_IMG_VIEW_ST, END_IMG_VIEW_ST,
                           START_IMG_MOVE_ST, WAIT_IMG_MOVE_ST, END_IMG_MOVE_ST);

   signal demo_fsm_st : demo_fsm_state;

   signal demo_start_reg1 : std_logic;
   signal demo_start_reg2 : std_logic;
   signal demo_start_reg  : std_logic;

   signal img_nr_cnt : std_logic_vector(3 downto 0);

begin

   -----------------------------------
   -- Registrazione di START_DEMO
   -- Creazione impulso DEMO_START_REG
   -----------------------------------
   DEMO_START_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         demo_start_reg1 <= '0';
         demo_start_reg2 <= '0';
         demo_start_reg  <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         demo_start_reg1 <= demo_start;
         demo_start_reg2 <= demo_start_reg1;
         demo_start_reg  <= (demo_start_reg1 and not demo_start_reg2);
      end if;
   end process DEMO_START_REG_PROC;

   ------------------------------------------------
   -- DEMO FSM
   -- 1) start della demo dopo fase accensione TFT
   -- 2) DEMO_SEL settato a '1' per emulare uC
   -- 3) copia di img numero 0 da SPI
   -- 4) set immagine numero 0 da visualizzare
   -- 5) DEMO_SEL settato a '0' per controllo uC
   ------------------------------------------------
   -- modifiche 18 luglio 2008
   -- 1) stato di busy del demo tft_rider
   ------------------------------------------------
   DEMO_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         demo_fsm_st    <= IDLE;
         img_start_move <= '0';
         img_view       <= '0';
         demo_sel       <= '0';
         img_nr_cnt     <= (others => '0');
         tft_surfer_busy <= '1';
      elsif (tft_clk = '1' and tft_clk'event) then
         -- default
         img_start_move <= '0';
         img_view       <= '0';
         case demo_fsm_st is
            when IDLE =>
               if (demo_start_reg = '1') then
                  demo_fsm_st    <= START_IMG_MOVE_ST;
                  img_start_move <= '1';
                  demo_sel       <= '1';
                  img_nr_cnt     <= (others => '0');
               end if;
            when START_IMG_MOVE_ST =>
               demo_fsm_st <= WAIT_IMG_MOVE_ST;
            when WAIT_IMG_MOVE_ST =>
               if (img_end_move = '1') then
                  demo_fsm_st <= END_IMG_MOVE_ST;
               end if;
            when END_IMG_MOVE_ST =>
               demo_fsm_st <= START_IMG_VIEW_ST;
               -- non voglio visualizzare l'avanzamento delle immagini !!!!
               img_view    <= '1'; -- view img_nr=img_nr_cnt
               --img_view    <= '0';
            when START_IMG_VIEW_ST =>
               demo_fsm_st <= WAIT_IMG_VIEW_ST;
-- debug baco SDRAM ?
            when WAIT_IMG_VIEW_ST =>
               if (img_nr_cnt = "0111") then
                  demo_fsm_st <= END_IMG_VIEW_ST;
                  --
                  tft_surfer_busy <= '0';
               else
                  demo_fsm_st    <= START_IMG_MOVE_ST;
                  img_start_move <= '1';
                  img_nr_cnt     <= img_nr_cnt + 1;
               end if;
-- debug baco SDRAM ?
--            when WAIT_IMG_VIEW_ST =>
--               demo_fsm_st    <= START_IMG_MOVE_ST;
--               img_start_move <= '1';
--               img_nr_cnt     <= img_nr_cnt + 1;
-- debug baco SDRAM ?
            when END_IMG_VIEW_ST =>
               demo_fsm_st <= IDLE;
               demo_sel    <= '0';
            when others =>
               demo_fsm_st <= IDLE;
               demo_sel    <= '0';
               img_nr_cnt  <= (others => '0');
               tft_surfer_busy <= '1';
         end case;
      end if;
   end process DEMO_FSM_PROC;

   img_nr <= img_nr_cnt;

end rtl_micro_emulator;
-----------------------------------------------------------------------