-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- MONSTER_MANAGER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- 07 sett: SYS_SDRAM_INIT sempre a 1??????????????????????????????????
--
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity monster_manager is
   port(
      -- system interface
      sys_rst            : in std_logic;
      sys_clk            : in std_logic;
      -- power_ctrl interface
      delay_100us        : in std_logic;
		-- SDRAM/monster_manager ready for read and write
		sys_sdram_ready_rw : out std_logic;
      -- sdram_ctrl interface
      sys_sdram_init     : out std_logic;
      sys_sdram_ready    : in std_logic;
      sys_r_wn           : out std_logic;
      sys_adsn           : out std_logic;
      -- system address definition
      -- bank(21-20) -> 4
      -- row(19-8)   -> 4096
      -- column(7-0) -> 256
      sys_addr           : out std_logic_vector(21 downto 0);
      sys_be             : out std_logic_vector(1 downto 0);
      -- write data
      sys_data_in        : out std_logic_vector(15 downto 0);
      -- read data and data valid
      sys_data_out       : in std_logic_vector(15 downto 0);
      sys_data_val       : in std_logic;
      -- read or write cycle is completed
      sys_cycle_end      : in std_logic;
      -- auto refresh
      sys_ref_req        : out std_logic;
      sys_ref_ack        : in std_logic;
      -- tft_timing_ctrl interface
		tft_row_start      : in std_logic;
      tft_sdram_dtmg     : in std_logic;
      tft_sdram_addr     : in std_logic_vector(21 downto 0);
		-- RGB 565
		tft_sdram_rgb_en   : out std_logic;
      tft_sdram_red      : out std_logic_vector(4 downto 0);
      tft_sdram_green    : out std_logic_vector(5 downto 0);
      tft_sdram_blue     : out std_logic_vector(4 downto 0);
      -- data_addr_fifo interface
      fifo_rd_req        : out std_logic;
      fifo_rd_add        : in std_logic_vector(21 downto 0); -- 22 bit
      fifo_rd_rgb        : in std_logic_vector(15 downto 0); -- 16 bit RGB 565
		fifo_rd_cnt        : in std_logic_vector(4 downto 0);
      fifo_rd_empty      : in std_logic;
		mode8_1            : in std_logic; -- selezione il modo di accesso
		-- (porta per permettere le letture)
		--fifo address
		wb_add_req         : out std_logic;
		wb_add             : in std_logic_vector(21 downto 0); -- 22 bit
		wb_add_empty       : in std_logic;
		--fifo data
		wb_data_we         : out std_logic;
		wb_data            : out std_logic_vector(15 downto 0)
   );
end monster_manager;
-----------------------------------------------------------------------
architecture rtl_monster_manager of monster_manager is

   type fsm_state is (IDLE, INIT, WAIT_CYCLE28,
	                   ST0, ST1, ST2, ST3, ST4, ST5, ST6,
							 ST7, ST8, ST9, ST10, ST11, ST12, ST13,
							 ST14, ST15, ST16, ST17, ST18, ST19, ST20,
							 ST21, ST22, ST23, ST24, ST25, ST26, ST27);

   signal fsm_st : fsm_state;

	-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	attribute syn_encoding : string;
	attribute syn_encoding of fsm_st : signal is "sequential";
	-- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   signal tft_row_start_reg : std_logic;

   signal tft_sdram_addr_reg : std_logic_vector(21 downto 0);

   signal delay_100us_reg : std_logic;

   signal fifo_empty_reg : std_logic;

   signal dummy_reg : std_logic_vector(7 downto 0);
   signal red_reg   : std_logic_vector(7 downto 0);
   signal green_reg : std_logic_vector(7 downto 0);
   signal blue_reg  : std_logic_vector(7 downto 0);

   signal fifo_rd_flag : std_logic;

   signal sys_data_val_reg1 : std_logic;
   signal sys_data_val_reg2 : std_logic;

   signal tft_red_reg   : std_logic_vector(7 downto 0);
   signal tft_green_reg : std_logic_vector(7 downto 0);
   signal tft_blue_reg  : std_logic_vector(7 downto 0);

   signal flag_wb_wr_8 : std_logic;
   signal flag_wb_wr_1 : std_logic;
   signal flag_wb_rd_1 : std_logic;
	
	signal wb_write_rd_data_en : std_logic;
	signal sys_data_val_reg : std_logic;

begin

   ---------------------------------
   -- TFT_ROW_START_REG_PROC process
   ---------------------------------
   TFT_ROW_START_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_row_start_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         tft_row_start_reg <= tft_row_start;
      end if;
   end process TFT_ROW_START_REG_PROC;

   ----------------------------------
   -- TFT_SDRAM_ADDR_REG_PROC process
   ----------------------------------
   -- TFT_SDRAM_ADDR_REG_PROC: process(sys_rst, sys_clk)
   -- begin
      -- if (sys_rst = '1') then
         -- tft_sdram_addr_reg <= (others => '0');
      -- elsif (sys_clk = '1' and sys_clk'event) then
         -- tft_sdram_addr_reg <= tft_sdram_addr;
      -- end if;
   -- end process TFT_SDRAM_ADDR_REG_PROC;

   -------------------------------
   -- DELAY_100US_REG_PROC process
   -------------------------------
   DELAY_100US_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         delay_100us_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         delay_100us_reg <= delay_100us;
      end if;
   end process DELAY_100US_REG_PROC;

   ------------------------------
   -- FIFO_EMPTY_REG_PROC process
   ------------------------------
   FIFO_EMPTY_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         fifo_empty_reg <= '1';
      elsif (sys_clk = '1' and sys_clk'event) then
         fifo_empty_reg <= fifo_rd_empty;
      end if;
   end process FIFO_EMPTY_REG_PROC;

   ----------------------
   -- FSM_MANAGER process
   ----------------------
   FSM_MONSTER_MANAGER_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         fsm_st         <= IDLE;
         sys_sdram_init <= '0';
         sys_adsn       <= '1';
         sys_r_wn       <= '1';
         sys_ref_req    <= '0';
         sys_addr       <= (others => '0');
         sys_be         <= (others => '0');
         sys_data_in    <= (others => '0');
         fifo_rd_req    <= '0';
         fifo_rd_flag   <= '0';
			sys_sdram_ready_rw <= '0';
			--
			wb_add_req     <= '0';
			flag_wb_wr_8   <= '0';
			flag_wb_wr_1   <= '0';
			flag_wb_rd_1   <= '0';
			--
      elsif (sys_clk = '1' and sys_clk'event) then
         --deafult
         sys_adsn    <= '1';
         sys_r_wn    <= '1';
         sys_be      <= (others => '0');
         sys_ref_req <= '0';
         fifo_rd_req <= '0';
			--
			wb_add_req  <= '0';
			--
         case fsm_st is
            when IDLE =>
               if (delay_100us_reg = '1') then
                  fsm_st <= INIT;
                  sys_sdram_init <= '1';
               end if;
            when INIT =>
               if (sys_sdram_ready = '1') then
                  fsm_st <= WAIT_CYCLE28;
               end if;
            when WAIT_CYCLE28 =>
               if (tft_row_start_reg = '1') then
                  -- 1) DTMG_SDRAM = 1 -> READ: sys_adsn = 0, sys_addr = indirizzo da tft_timing_ctrl
                  -- 2) DTMG_SDRAM = 0 -> AUTO REFRESH: sys_ref_req = 1, sys_addr = don't care
                  -- Per evitare di introdurre un mux su sys_addr in funzione del valore di DTMG
                  -- utilizzo lo stesso assegnamento di sys_addr del READ anche per AUTO REFRESH
                  fsm_st <= ST0;
                  sys_addr    <= tft_sdram_addr;
--                  if (tft_sdram_dtmg = '1') then
--                     -- READ
--                     sys_adsn <= '0';
--                  else
--                     -- AUTO REFRESH
--                     sys_ref_req <= '1';
--                  end if;
                  sys_adsn    <= (not tft_sdram_dtmg);
                  sys_ref_req <= (not tft_sdram_dtmg);
						-- 17 sett 2011: cycle 28 states is now running
						-- and monster_manager is ready for read and write
						sys_sdram_ready_rw <= '1';
               end if;
            when ST0 =>
               -- SDRAM: READ delay dovuto alla registrazione di SDRAM_CTRL
               fsm_st <= ST1;
            when ST1 =>
               -- SDRAM: READ con AUTOPRECHARGE, ACTIVE row/bank
               fsm_st <= ST2;
            when ST2 =>
               -- SDRAM: NOP 
               fsm_st <= ST3;
            when ST3 =>
               -- SDRAM: READ con AUTOPRECHARGE, column/bank
               fsm_st <= ST4;
            when ST4 =>
               -- SDRAM: READ CAS 1 sys_clk
               fsm_st <= ST5;
            when ST5 =>
               -- SDRAM: READ CAS 2 sys_clk
               fsm_st <= ST6;
            when ST6 =>
               -- SDRAM: READ CAS 3 sys_clk, READ DQ burst 0
               fsm_st <= ST7;
            when ST7 =>
               -- SDRAM: READ DQ burst 1
               fsm_st <= ST8;
            when ST8 =>
               -- SDRAM: READ DQ burst 2
               fsm_st <= ST9;

            when ST9 =>
               -- SDRAM: READ DQ burst 3
               fsm_st <= ST10;
					-- SDRAM: preparazione operazione di WRITE
					if (fifo_empty_reg = '0') then
						if ((fifo_rd_cnt >= 8) and (mode8_1 = '1')) then
							flag_wb_wr_8 <= '1';
						elsif ((fifo_rd_cnt >= 1) and (mode8_1 = '0')) then
							flag_wb_wr_1 <= '1';
						end if;
					-- SDRAM: preparazione operazione di READ
					elsif (wb_add_empty = '0') then
							flag_wb_rd_1 <= '1';
					end if;

            when ST10 =>
               -- SDRAM: READ DQ burst 4
               fsm_st <= ST11;
               if ((flag_wb_wr_8 = '1') or (flag_wb_wr_1 = '1')) then
                  -- WRITE: lettura di addr_fifo e data_fifo
                  fifo_rd_flag <= '1';
                  fifo_rd_req  <= '1'; -- FIFO READ pixel 0
					elsif (flag_wb_rd_1 = '1') then
						wb_add_req <= '1';
               end if;

            when ST11 =>
               -- SDRAM: READ DQ burst 5
               fsm_st <= ST12;
               if (flag_wb_wr_8 = '1') then
                  fifo_rd_req <= '1'; -- FIFO READ pixel 1
               end if;

				when ST12 =>
               -- SDRAM: READ DQ burst 6
               fsm_st <= ST13;
               if (flag_wb_wr_8 = '1') then
                  fifo_rd_req <= '1'; -- FIFO READ pixel 2
               end if;

				when ST13 =>
               -- SDRAM: READ DQ burst 7
               fsm_st <= ST14;
               if ((flag_wb_wr_8 = '1') or (flag_wb_wr_1 = '1')) then
                  fifo_rd_req <= flag_wb_wr_8; -- FIFO READ pixel 3 solo se flag_wb_wr_8 = '1' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                  -- WRITE:
                  -- sys_adsn = 0, sys_r_wn = 0, sys_addr = indirizzo da addr_fifo, sys_data = dati da data_fifo:
                  sys_adsn    <= '0';
                  sys_r_wn    <= '0';
                  sys_addr    <= fifo_rd_add;
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 0
					elsif (flag_wb_rd_1 = '1') then
                  -- READ: sys_adsn = 0, sys_r_wn = 1, sys_addr = indirizzo da wb_rd_fifo
                  sys_adsn    <= '0';
                  sys_r_wn    <= '1';
                  sys_addr    <= wb_add;
               end if;

            when ST14 =>
               fsm_st <= ST15;
               if (flag_wb_wr_8 = '1') then
						fifo_rd_req <= '1'; -- FIFO READ pixel 4
                  -- SDRAM: WRITE delay dovuto alla registrazione di SDRAM_CTRL
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 1
					elsif (flag_wb_wr_1 = '1') then
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

            when ST15 =>
               fsm_st <= ST16;
               if (flag_wb_wr_8 = '1') then
						fifo_rd_req <= '1'; -- FIFO READ pixel 5
                  -- SDRAM: WRITE con AUTOPRECHARGE, ACTIVE row/bank
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 2
					elsif (flag_wb_wr_1 = '1') then
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

				when ST16 =>
               fsm_st <= ST17;
               if (flag_wb_wr_8 = '1') then
						fifo_rd_req <= '1'; -- FIFO READ pixel 6
                  -- SDRAM: WRITE NOP
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 3
					elsif (flag_wb_wr_1 = '1') then
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

				when ST17 =>
               fsm_st <= ST18;
               if (flag_wb_wr_8 = '1') then
						fifo_rd_req <= '1'; -- FIFO READ pixel 7
                  --SDRAM: WRITE con AUTOPRECHARGE, column/bank, DQ burst 0
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 4
					elsif (flag_wb_wr_1 = '1') then
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

            when ST18 =>
               fsm_st <= ST19;
               if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 1
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 5
					elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 1 BYTE DISABLE
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

            when ST19 =>
               fsm_st <= ST20;
               if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 2
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 6
					elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 2 BYTE DISABLE
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

            when ST20 =>
               fsm_st <= ST21;
               if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 3
                  sys_data_in <= fifo_rd_rgb(15 downto 0); -- SDRAM CTRL pixel 7
					elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 3 BYTE DISABLE
						--fifo_rd_req <= '0';
                  sys_data_in <= (others => '0');
						sys_be      <= (others => '1');
               end if;

            when ST21 =>
               fsm_st <= ST22;
               --if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 4
					-- elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 4 BYTE DISABLE
               --end if;
            when ST22 =>
               fsm_st <= ST23;
               --if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 5
					-- elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 5 BYTE DISABLE
               --end if;
            when ST23 =>
               fsm_st <= ST24;
               --if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 6
					-- elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 6 BYTE DISABLE
               --end if;
            when ST24 =>
               fsm_st <= ST25;
               --if (flag_wb_wr_8 = '1') then
                  -- SDRAM: WRITE DQ burst 7
					-- elsif (flag_wb_wr_1 = '1') then
                  -- SDRAM: WRITE DQ burst 7 BYTE DISABLE
               --end if;
            when ST25 =>
               fsm_st <= ST26;
               --if ((flag_wb_wr_8 = '1') or (flag_wb_wr_1 = '1')) then
                  -- SDRAM: WRITE NOP
               --end if;
            when ST26 =>
               fsm_st <= ST27;
               --if ((flag_wb_wr_8 = '1') or (flag_wb_wr_1 = '1')) then
                  -- SDRAM: WRITE NOP
               --end if;
            when ST27 =>
               fsm_st <= ST0;
               -- reset flag di lettura (WRITE SDRAM) di data_addr_fifo
               fifo_rd_flag <= '0';
					flag_wb_wr_8 <= '0';
					flag_wb_wr_1 <= '0';
					flag_wb_rd_1 <= '0';
					--
               -- 1) DTMG_SDRAM = 1 -> READ: sys_adsn = 0, sys_addr = indirizzo da tft_timing_ctrl
               -- 2) DTMG_SDRAM = 0 -> AUTO REFRESH: sys_ref_req = 1, sys_addr = don't care
               -- Per evitare di introdurre un mux su sys_addr in funzione del valore di DTMG
               -- utilizzo lo stesso assegnamento di sys_addr del READ anche per AUTO REFRESH
               sys_addr     <= tft_sdram_addr;
--               if (tft_sdram_dtmg = '1') then
--                  -- READ
--                  sys_adsn <= '0';
--               else
--                  -- AUTO REFRESH
--                  sys_ref_req <= '1';
--               end if;
               sys_adsn     <= (not tft_sdram_dtmg);
               sys_ref_req  <= (not tft_sdram_dtmg);
            when others =>
               fsm_st <= IDLE;
               sys_sdram_init <= '0';
               sys_adsn       <= '1';
               sys_r_wn       <= '1';
               sys_addr       <= (others => '0');
               sys_be         <= (others => '0');
               sys_data_in    <= (others => '0');
               fifo_rd_req    <= '0';
					sys_sdram_ready_rw <= '0';
					--
					flag_wb_wr_8 <= '0';
					flag_wb_wr_1 <= '0';
					flag_wb_rd_1 <= '0';
					--
         end case;
      end if;
   end process FSM_MONSTER_MANAGER_PROC;

   ---------------------------------
   -- TFT_SDRAM_RGB_REG_PROC process
   ---------------------------------
	-- attenzione controllare se mettere DATA VALID (feedback !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)

   TFT_SDRAM_RGB_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_sdram_red   <= (others => '0');
         tft_sdram_green <= (others => '0');
         tft_sdram_blue  <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
         --if (sys_data_val = '1') then
         --if ((sys_data_val = '1') and (flag_wb_wr_8 = '0')) then
         if ((sys_data_val = '1') and (flag_wb_rd_1 = '0')) then
            tft_sdram_red   <= sys_data_out(15 downto 11);
            tft_sdram_green <= sys_data_out(10 downto 5);
            tft_sdram_blue  <= sys_data_out(4 downto 0);
         end if;
      end if;
   end process TFT_SDRAM_RGB_REG_PROC;

   --------------------------------
   -- TFT_SDRAM_RGB_EN_PROC process
   --------------------------------
   TFT_SDRAM_RGB_EN_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_sdram_rgb_en <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         --tft_sdram_rgb_en <= sys_data_val;
         --tft_sdram_rgb_en <= sys_data_val and (not flag_wb_wr_8); OK, funziona
         tft_sdram_rgb_en <= sys_data_val and (not flag_wb_rd_1);
      end if;
   end process TFT_SDRAM_RGB_EN_PROC;

   --------------------------------
   -- SYS_DATA_VAL_REG_PROC process
   --------------------------------
   SYS_DATA_VAL_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_data_val_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_data_val_reg <= sys_data_val;
      end if;
   end process SYS_DATA_VAL_REG_PROC;

	wb_write_rd_data_en <= '1' when ((sys_data_val = '1') and (sys_data_val_reg = '0') and (flag_wb_rd_1 = '1')) else '0';

   ---------------------
   -- WISHBONE READ FIFO
	-- WB_READ_FIFO_PROC
   ---------------------
   WB_READ_FIFO_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         wb_data_we <= '0';
         wb_data    <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
         wb_data_we <= '0';
         --if ((sys_data_val = '1') and (flag_wb_rd_1 = '1')) then
         if (wb_write_rd_data_en = '1') then
				wb_data_we <= '1';
				wb_data    <= sys_data_out;
         end if;
      end if;
   end process WB_READ_FIFO_PROC;

end rtl_monster_manager;
-----------------------------------------------------------------------