-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- PIXEL_ADDR_DATA_FIFO.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
-----------------------------------------------------------------------
entity pixel_addr_data_fifo is
   port(
      -- sys interface
      sys_rst     : in std_logic;
      sys_clk     : in std_logic; -- system clock for READ (SDRAM clk)
      sys_div_clk : in std_logic; -- WRITE clock (uC interface)
      -- WRITE interface
      fifo_wr_req : in std_logic;
      fifo_wr_add : in std_logic_vector(21 downto 0);
      fifo_wr_rgb : in std_logic_vector(15 downto 0);
      fifo_full   : out std_logic;
      -- READ interface
      fifo_rd_req : in std_logic;
      fifo_rd_add : out std_logic_vector(21 downto 0);
      fifo_rd_rgb : out std_logic_vector(15 downto 0);
	  fifo_rd_cnt : out std_logic_vector(4 downto 0);
      fifo_empty  : out std_logic
   );
end pixel_addr_data_fifo;
-----------------------------------------------------------------------
architecture rtl_pixel_addr_data_fifo of pixel_addr_data_fifo is

   component pixel_addr_fifo
      port(
			Data    : in std_logic_vector(21 downto 0); 
			WrClock : in std_logic; 
			RdClock : in std_logic; 
			WrEn    : in std_logic; 
			RdEn    : in std_logic; 
			Reset   : in std_logic; 
			RPReset : in std_logic; 
			Q       : out std_logic_vector(21 downto 0); 
			Empty   : out std_logic; 
			Full    : out std_logic
      );
   end component;

   component pixel_data_fifo
      port(
			Data    : in  std_logic_vector(15 downto 0); 
			WrClock : in  std_logic; 
			RdClock : in  std_logic; 
			WrEn    : in  std_logic; 
			RdEn    : in  std_logic; 
			Reset   : in  std_logic; 
			RPReset : in  std_logic; 
			Q       : out  std_logic_vector(15 downto 0); 
			RCNT    : out  std_logic_vector(4 downto 0); 
			Empty   : out  std_logic; 
			Full    : out  std_logic
      );
   end component;

begin

   PIXEL_ADDR_FIFO_INST: pixel_addr_fifo
      port map(
			Data    => fifo_wr_add,
			WrClock => sys_div_clk,
			RdClock => sys_clk,
			WrEn    => fifo_wr_req,
			RdEn    => fifo_rd_req,
			Reset   => sys_rst,
			RPReset => sys_rst,
			Q       => fifo_rd_add,
			Empty   => open,
			Full    => open
      );

   PIXEL_DATA_FIFO_INST: pixel_data_fifo
      port map(
			Data    => fifo_wr_rgb,
			WrClock => sys_div_clk,
			RdClock => sys_clk,
			WrEn    => fifo_wr_req,
			RdEn    => fifo_rd_req,
			Reset   => sys_rst,
			RPReset => sys_rst,
			Q       => fifo_rd_rgb,
			RCNT    => fifo_rd_cnt,
			Empty   => fifo_empty,
			Full    => fifo_full
      );

end rtl_pixel_addr_data_fifo;
-----------------------------------------------------------------------