-- ====================================================================
-- SDRAM_PKG.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver 0.1 - date 17 Apr 2007
--             ver 1.0 - date 04 May 2007
--             ver 1.0 - date 03 Jun 2007
-- ====================================================================
-- Note:
-- 1) CAS latcency = 2
--
-- 2) BURST length = 8
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
-----------------------------------------------------------------------
package sdram_pkg is

   -- TDLY for functional simulation
   constant TDLY : time := 2 ns;
   
   -- SDRAM mode register setting
   constant MR_WRITE_BURST  : std_logic := '0';                     -- programmed lenght
   constant MR_OPERATION    : std_logic_vector(1 downto 0) := "00";  -- standard
   -- latency 3 = "011"
   -- latency 2 = "010"
   constant MR_CAS_LATENCY  : std_logic_vector(2 downto 0) := "011";
   -- sequential = 0
   constant MR_BURST_TYPE   : std_logic := '0';
   constant MR_BURST_LENGTH : std_logic_vector(2 downto 0) := "011"; -- burst 8

   -- SDRAM comands (sdram_csn, sdram_rasn, sdram_casn, sdram_wen)
   constant C_INHIBIT       : std_logic_vector(3 downto 0) := "1111";
   constant C_NOP           : std_logic_vector(3 downto 0) := "0111";
   constant C_ACTIVE        : std_logic_vector(3 downto 0) := "0011";
   constant C_READ          : std_logic_vector(3 downto 0) := "0101";
   constant C_WRITE         : std_logic_vector(3 downto 0) := "0100";
   constant C_BURST_END     : std_logic_vector(3 downto 0) := "0110";
   constant C_PRECHARGE     : std_logic_vector(3 downto 0) := "0010";
   constant C_AUTO_REFRESH  : std_logic_vector(3 downto 0) := "0001";
   constant C_LOAD_MODE_REG : std_logic_vector(3 downto 0) := "0000";

end sdram_pkg;
-----------------------------------------------------------------------