-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- SDRAM_TEST_GENERATOR.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
library work;
use work.tft_pkg.all;
-----------------------------------------------------------------------
entity sdram_test_generator is
   port(
      -- system interface
      sys_rst           : in std_logic;
      tft_clk           : in std_logic;
      -- monster_manager interface
      sdram_ready_rw    : in std_logic;
      -- sdram fifo write interface
      sdram_fifo_wr_req : out std_logic;
      sdram_fifo_wr_add : out std_logic_vector(21 downto 0);
      sdram_fifo_wr_rgb : out std_logic_vector(15 downto 0)
   ); 
end sdram_test_generator;
-----------------------------------------------------------------------
architecture rtl_sdram_test_generator of sdram_test_generator is
	
	signal sdram_ready_rw_reg1 : std_logic;
	signal sdram_ready_rw_reg2 : std_logic;

   type fsm_state is (IDLE, ST_SDRAM_READY, ST_WAIT_START, ST_WRITE);

   signal fsm_st : fsm_state;
	
   signal test_rst : std_logic;
		
   signal cnt_rst : std_logic;

   signal scaler_cnt : std_logic_vector(2 downto 0);

   signal pixel_cnt     : std_logic_vector(18 downto 0); -- 19 bit!!!
   signal column_cnt    : std_logic_vector(9 downto 0);
   signal row_cnt       : std_logic_vector(9 downto 0);
   signal bank_sect_cnt : std_logic_vector(2 downto 0);

begin

   ----------------------------------
   -- SDRAM_READY_RW_REG_PROC process
   ----------------------------------
   SDRAM_READY_RW_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			sdram_ready_rw_reg1 <= '0';
			sdram_ready_rw_reg2 <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			sdram_ready_rw_reg1 <= sdram_ready_rw;
			sdram_ready_rw_reg2 <= sdram_ready_rw_reg1;
		end if;
   end process SDRAM_READY_RW_REG_PROC;

   -------------------
   -- FSM_PROC process
   -------------------
	FSM_PROC: process(sys_rst, tft_clk)
	begin
      if (sys_rst = '1') then
         fsm_st   <= IDLE;
         test_rst <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         case fsm_st is
            when IDLE =>
               if (sdram_ready_rw_reg2 = '1') then
                  fsm_st   <= ST_SDRAM_READY;
               end if;
            when ST_SDRAM_READY =>
					fsm_st <= ST_WAIT_START;
            when ST_WAIT_START =>
               if (sdram_ready_rw_reg2 = '1') then
                  fsm_st   <= ST_WRITE;
						test_rst <= '1';
               end if;
            when ST_WRITE =>
               fsm_st   <= ST_WRITE;
				when others =>
					fsm_st   <= IDLE;
					test_rst <= '0';
			end case;
		end if;
   end process FSM_PROC;

   cnt_rst <= (sys_rst or (not test_rst));

   --------------------------
   -- SCALER_CNT_PROC process
   --------------------------
   SCALER_CNT_PROC: process(cnt_rst, tft_clk)
   begin
      if (cnt_rst = '1') then
			scaler_cnt <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			scaler_cnt <= scaler_cnt + 1;
		end if;
   end process SCALER_CNT_PROC;

   -----------------------
   -- COUNTER_PROC process
   -----------------------
   COUNTER_PROC: process(cnt_rst, tft_clk)
   begin
      if (cnt_rst = '1') then
			bank_sect_cnt <= (others => '0');
         pixel_cnt     <= (others => '0');
         column_cnt    <= (others => '0');
         row_cnt       <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			if ( (scaler_cnt(0) = '1') and (sdram_ready_rw_reg2 = '1') ) then
				-- PIXEL and BANK SECT increment 
				if (pixel_cnt = (TOTAL_PIXEL - 1)) then
					bank_sect_cnt <= bank_sect_cnt + 1;
					pixel_cnt     <= (others => '0');
				else
					pixel_cnt <= pixel_cnt + 1;
				end if;
				-- COLUMN and ROW increment
				if (column_cnt = (H_PIXELS - 1)) then
					column_cnt <= (others => '0');
					if (row_cnt = (V_LINES - 1)) then
						row_cnt <= (others => '0');
					else
						row_cnt <= row_cnt + 1;
					end if;
				else
					column_cnt <= column_cnt + 1;
				end if;
			end if;
		end if;
   end process COUNTER_PROC;

   --------------------------------------
   -- SDRAM_FIFO_WR_REQ_ADDR_PROC process
   --------------------------------------
   SDRAM_FIFO_REQ_ADDR_PROC: process(cnt_rst, tft_clk)
   begin
      if (cnt_rst = '1') then
			sdram_fifo_wr_req <= '0';
			sdram_fifo_wr_add <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			sdram_fifo_wr_req <= (scaler_cnt(0) and sdram_ready_rw_reg2);
			--sdram_fifo_wr_req <= '0';
			sdram_fifo_wr_add <= (bank_sect_cnt & pixel_cnt);
		end if;
   end process SDRAM_FIFO_REQ_ADDR_PROC;

   ---------------------------------
   -- SDRAM_FIFO_WR_RGB_PROC process
   ---------------------------------
   SDRAM_FIFO_WR_RGB_PROC: process(cnt_rst, tft_clk)
   begin
      if (cnt_rst = '1') then
         sdram_fifo_wr_rgb <= (others => '1');
      elsif (tft_clk = '1' and tft_clk'event) then
			----------------
			-- TEST BAR mode
			----------------
         -- if ( ( (row_cnt(2 downto 0) = 0) or (row_cnt = (V_LINES - 1)) ) or
              -- ( (column_cnt(2 downto 0) = 0) or (column_cnt = (H_PIXELS - 1)) ) ) then
         -- if ( (row_cnt(0) = '0') or (column_cnt(0) = '0') ) then
         -- if ( (column_cnt(0) = '0') ) then
         -- if ( ( (row_cnt(1 downto 0) = 0) or (row_cnt = (V_LINES - 1)) ) or
              -- ( (column_cnt(1 downto 0) = 0) or (column_cnt = (H_PIXELS - 1)) ) ) then
         if (row_cnt(1 downto 0) = column_cnt(1 downto 0)) then
            sdram_fifo_wr_rgb <= ("11111" & "000000" & "00000");
         else
            sdram_fifo_wr_rgb <= ("00000" & "000000" & "11111");
         end if;
			----------------
			-- TEST BAR mode
			----------------
			--------------------------
			-- TEST PIXEL_COUNTER mode
			--------------------------
			--sdram_fifo_wr_rgb <= pixel_cnt(15 downto 0);
			--------------------------
			-- TEST PIXEL_COUNTER mode
			--------------------------
      end if;
   end process SDRAM_FIFO_WR_RGB_PROC;

end rtl_sdram_test_generator;
-----------------------------------------------------------------------