-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- TFT_ON_OFF_BUTTON_CTRL.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity tft_on_off_button_ctrl is
   port(
      sys_rst           : in std_logic;
		osch_clk          : in std_logic;
      tft_on_off_button : in std_logic; -- pull-up, active low
      tft_on            : out std_logic; -- osch_clk domaain
      tft_off           : out std_logic; -- osch_clk domaain
      test_button       : out std_logic -- led test
   );
end tft_on_off_button_ctrl;
-----------------------------------------------------------------------
architecture rtl_tft_on_off_button_ctrl of tft_on_off_button_ctrl is

   signal button_reg : std_logic_vector(7 downto 0);

   signal button_imp_sig  : std_logic;
   signal button_imp_reg0 : std_logic;
   signal button_imp_reg1 : std_logic;

   signal button_pre_sig : std_logic;
   signal button_pre_reg : std_logic;

   type button_state is (IDLE, FIRST_PRE, SECOND_PRE);
   
   signal button_st : button_state;

begin

   --------------------------
   -- BUTTON_REG_PROC process
   --------------------------
   BUTTON_REG_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         button_reg <= (others => '1');
      elsif (osch_clk = '1' and osch_clk'event) then
         button_reg(0)          <= tft_on_off_button;
         button_reg(7 downto 1) <= button_reg(6 downto 0);
      end if;
   end process BUTTON_REG_PROC;

   -- button is pressed then constant zero on button input
   button_imp_sig <= '1' when (button_reg = "00000000") else
                     '0';

   ------------------------------
   -- BUTTON_IMP_REG_PROC process
   ------------------------------
   BUTTON_IMP_REG_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         button_imp_reg0 <= '0';
         button_imp_reg1 <= '0';
      elsif (osch_clk = '1' and osch_clk'event) then
         button_imp_reg0 <= button_imp_sig;
         button_imp_reg1 <= button_imp_reg0;
      end if;
   end process BUTTON_IMP_REG_PROC;

   -- as soon as button is pressed a single impulse of osch_clk period is generated
   button_pre_sig <= '1' when ((button_imp_reg0 = '1') and (button_imp_reg1 = '0')) else
                     '0';

   ------------------------------
   -- BUTTON_PRE_REG_PROC process
   ------------------------------
   BUTTON_PRE_REG_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         button_pre_reg <= '0';
      elsif (osch_clk = '1' and osch_clk'event) then
         button_pre_reg <= button_pre_sig;
      end if;
   end process BUTTON_PRE_REG_PROC;

   ------------------------------
   -- TFT_ON_OFF_FSM_PROC process
   ------------------------------
   TFT_ON_OFF_FSM_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         button_st   <= IDLE;
         tft_on      <= '0';
         tft_off     <= '0';
         test_button <= '0';
      elsif (osch_clk = '1' and osch_clk'event) then
         -- default
         tft_on  <= '0';
         tft_off <= '0';
         case button_st is
            when IDLE =>
               if (button_pre_reg = '1') then
                  button_st   <= FIRST_PRE;
                  tft_on      <= '1';
                  test_button <= '1';
               end if;
            when FIRST_PRE =>
               if (button_pre_reg = '1') then
                  button_st   <= SECOND_PRE;
                  tft_off     <= '1';
                  test_button <= '0';
               end if;
            when SECOND_PRE =>
               button_st <= IDLE;
            when others =>
               button_st   <= IDLE;
               test_button <= '0';
         end case;
      end if;
   end process TFT_ON_OFF_FSM_PROC;

end rtl_tft_on_off_button_ctrl;
-----------------------------------------------------------------------