-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- TFT_PIXEL_RGB_PIPELINER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
-----------------------------------------------------------------------
entity tft_pixel_rgb_pipeliner is
   port(
      -- sys interface
      sys_rst             : in std_logic;
      sys_clk             : in std_logic; -- WRITE (monster_manager)
      tft_clk             : in std_logic; -- READ (tft_timing_ctrl)
      -- WRITE: monster_manager interface
		tft_sdram_rgb_en    : in std_logic;
      tft_sdram_red_in    : in std_logic_vector(4 downto 0); -- 5 bit
      tft_sdram_green_in  : in std_logic_vector(5 downto 0); -- 6 bit
      tft_sdram_blue_in   : in std_logic_vector(4 downto 0); -- 5 bit
      -- READ: tft_timing_ctrl interface
      tft_sdram_red_out   : out std_logic_vector(4 downto 0); -- 5 bit
      tft_sdram_green_out : out std_logic_vector(5 downto 0); -- 6 bit
      tft_sdram_blue_out  : out std_logic_vector(4 downto 0)  -- 5 bit
   );
end tft_pixel_rgb_pipeliner;
-----------------------------------------------------------------------
architecture rtl_tft_pixel_rgb_pipeliner of tft_pixel_rgb_pipeliner is

   component pixel_rgb_fifo
      port(
			Data    : in  std_logic_vector(15 downto 0); 
			WrClock : in  std_logic; 
			RdClock : in  std_logic; 
			WrEn    : in  std_logic; 
			RdEn    : in  std_logic; 
			Reset   : in  std_logic; 
			RPReset : in  std_logic; 
			Q       : out  std_logic_vector(15 downto 0); 
			Empty   : out  std_logic; 
			Full    : out  std_logic
      );
   end component;

	signal tft_sdram_rgb_en_reg1 : std_logic;
	signal tft_sdram_rgb_en_reg2 : std_logic;

   type read_enable_state is (IDLE, READ_ENABLE);

   signal read_enable_st : read_enable_state;

	signal read_en : std_logic;

	signal pixel_rgb_fifo_in  : std_logic_vector(15 downto 0);
	signal pixel_rgb_fifo_out : std_logic_vector(15 downto 0);

begin

   ------------------------------------
   -- TFT_SDRAM_RGB_EN_REG_PROC process
   ------------------------------------
   TFT_SDRAM_RGB_EN_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         tft_sdram_rgb_en_reg1 <= '0';
         tft_sdram_rgb_en_reg2 <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         tft_sdram_rgb_en_reg1 <= tft_sdram_rgb_en;
         tft_sdram_rgb_en_reg2 <= tft_sdram_rgb_en_reg1;
      end if;
   end process TFT_SDRAM_RGB_EN_REG_PROC;

   -------------------------------
   -- READ_ENABLE_FSM_PROC process
   -------------------------------
   READ_ENABLE_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         read_enable_st <= IDLE;
         read_en        <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			case read_enable_st is
				when IDLE =>
					if (tft_sdram_rgb_en_reg2 = '1') then
						read_enable_st <= READ_ENABLE;
						read_en        <= '1';
					end if;
				when READ_ENABLE =>
					read_enable_st <= READ_ENABLE;
					read_en        <= '1';
				when others =>
					read_enable_st <= IDLE;
					read_en        <= '0';
			end case;
		end if;
   end process READ_ENABLE_FSM_PROC;

	pixel_rgb_fifo_in  <= (tft_sdram_red_in & tft_sdram_green_in & tft_sdram_blue_in);

   PIXEL_RGB_FIFO_INST: pixel_rgb_fifo
      port map(
			Data    => pixel_rgb_fifo_in,
			WrClock => sys_clk,
			RdClock => tft_clk,
			WrEn    => tft_sdram_rgb_en,
			RdEn    => read_en,
			Reset   => sys_rst,
			RPReset => sys_rst,
			Q       => pixel_rgb_fifo_out,
			Empty   => open,
			Full    => open
      );

	tft_sdram_red_out   <= pixel_rgb_fifo_out(15 downto 11);
	tft_sdram_green_out <= pixel_rgb_fifo_out(10 downto 5);
	tft_sdram_blue_out  <= pixel_rgb_fifo_out(4 downto 0);

end rtl_tft_pixel_rgb_pipeliner;
-----------------------------------------------------------------------