-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- TFT_PKG.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
-- 
-- VERTICAL SYNC
--
--    ...--- Blanking LINES -->|<-- Active LINES -->|<--...
--                             |                    |            
--                             |                    |            
-- ...--+          +-------------------- ... -------------------+
--      |   VSYNC  |    VBP    |      V Pixels      |    VFP    |
--      |<--V Sync |<--V Back  |<---- TFT Rows ---->|<--V Front |
--      |   Time-->|   Porch-->|                        Porch-->|
--      +----------+                                            +--...
--      |                       V PERIOD                        |
--      |<---------------------- FRAME ------------------------>|
-- 
-- 
-- HORIZONTAL SYNC
--
--    ...-- Blanking PIXELS -->|<-- Active PIXELS ->|<--...
--                             |                    |            
--                             |                    |            
-- ...--+          +-------------------- ... -------------------+
--      |   HSYNC  |    HBP    |      H Pixels      |    HFP    |
--      |<--H Sync |<--H Back  |<--- TFT Columns -->|<--H Front |
--      |   Time-->|   Porch-->|                        Porch-->|
--      +----------+                                            +--...
--      |                       H PERIOD                        |
--      |<--------------------- TFT ROW ----------------------->|
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
-----------------------------------------------------------------------
package tft_pkg is

	-------------------------------------
	-- TFT parameters only for simulation
	-------------------------------------
   -- -- horizontal timing parameters
   -- constant H_PIXELS     : INTEGER := 32;
   -- constant H_FRONTPORCH : INTEGER := 4;
   -- constant H_SYNCTIME   : INTEGER := 1;
   -- constant H_BACKPORCH  : INTEGER := 3;
   -- constant H_PERIOD     : INTEGER := H_SYNCTIME + H_PIXELS + H_FRONTPORCH + H_BACKPORCH;

   -- -- vertical timing parameters
   -- constant V_LINES      : INTEGER := 4;
   -- constant V_FRONTPORCH : INTEGER := 3;
   -- constant V_SYNCTIME   : INTEGER := 1;
   -- constant V_BACKPORCH  : INTEGER := 2;
   -- constant V_PERIOD     : INTEGER := V_SYNCTIME + V_LINES + V_FRONTPORCH + V_BACKPORCH;

	-- -- pixel
	-- constant TOTAL_PIXEL  : INTEGER := 128;

	-------------------------------
	-- VESA, SVGA 800 x 600, 40 MHz
	-------------------------------
   -- -- horizontal timing parameters
   -- constant H_PIXELS     : INTEGER := 800;
   -- constant H_FRONTPORCH : INTEGER := 40;
   -- constant H_SYNCTIME   : INTEGER := 128;
   -- constant H_BACKPORCH  : INTEGER := 88;
   -- constant H_PERIOD     : INTEGER := H_SYNCTIME + H_PIXELS + H_FRONTPORCH + H_BACKPORCH; -- 1056

   -- -- vertical timing parameters
   -- constant V_LINES      : INTEGER := 600;
   -- constant V_FRONTPORCH : INTEGER := 1;
   -- constant V_SYNCTIME   : INTEGER := 4;
   -- constant V_BACKPORCH  : INTEGER := 23;
   -- constant V_PERIOD     : INTEGER := V_SYNCTIME + V_LINES + V_FRONTPORCH + V_BACKPORCH; -- 628

	-- -- pixel
	-- constant TOTAL_PIXEL  : INTEGER := 480000;

	---------------------
	-- 640x480 @25 MHz OK
	---------------------
   -- -- horizontal timing parameters
   -- constant H_PIXELS     : INTEGER := 640;
   -- constant H_FRONTPORCH : INTEGER := 16;
   -- constant H_SYNCTIME   : INTEGER := 96;
   -- constant H_BACKPORCH  : INTEGER := 48;
   -- constant H_PERIOD     : INTEGER := H_SYNCTIME + H_PIXELS + H_FRONTPORCH + H_BACKPORCH; -- 800

   -- -- vertical timing parameters
   -- constant V_LINES      : INTEGER := 480;
   -- constant V_FRONTPORCH : INTEGER := 10;
   -- constant V_SYNCTIME   : INTEGER := 2;
   -- constant V_BACKPORCH  : INTEGER := 33;
   -- constant V_PERIOD     : INTEGER := V_SYNCTIME + V_LINES + V_FRONTPORCH + V_BACKPORCH; -- 525

	-- -- pixel
	-- constant TOTAL_PIXEL  : INTEGER := 307200;

---------------------------
-- 800x600 @40 MHz 37.5 MHz
---------------------------
-- constant HTOTAL : integer := 1088;
-- constant HSYNC : integer := 128;
-- constant HBACK_PORCH : integer := 128;
-- constant HACTIVE : integer := 800;
-- constant HFRONT_PORCH : integer := 32;
-- constant VTOTAL : integer := 619;
-- constant VSYNC : integer := 4;
-- constant VBACK_PORCH : integer := 14;
-- constant VACTIVE : integer := 600;
-- constant VFRONT_PORCH : integer := 1;
-- constant RECTANGLE_HSTART : integer := 100;
-- constant RECTANGLE_HEND : integer := 540;
-- constant RECTANGLE_VSTART : integer := 100;
-- constant RECTANGLE_VEND : integer := 380;

	---------------------------
	-- 800x600 @40 MHz 37.5 MHz OK!!!!!!!!!!!!!!!!!!!!
	---------------------------
   -- -- horizontal timing parameters
   -- constant H_PIXELS     : INTEGER := 800;
   -- constant H_FRONTPORCH : INTEGER := 32;
   -- constant H_SYNCTIME   : INTEGER := 128;
   -- constant H_BACKPORCH  : INTEGER := 128;
   -- constant H_PERIOD     : INTEGER := H_SYNCTIME + H_PIXELS + H_FRONTPORCH + H_BACKPORCH; -- 1088

   -- -- vertical timing parameters
   -- constant V_LINES      : INTEGER := 600;
   -- constant V_FRONTPORCH : INTEGER := 1;
   -- constant V_SYNCTIME   : INTEGER := 4;
   -- constant V_BACKPORCH  : INTEGER := 14;
   -- constant V_PERIOD     : INTEGER := V_SYNCTIME + V_LINES + V_FRONTPORCH + V_BACKPORCH; -- 619

	-- -- pixel
	-- constant TOTAL_PIXEL  : INTEGER := 480000;

	   -- horizontal timing parameters
   constant H_PIXELS     : INTEGER := 800;
   constant H_FRONTPORCH : INTEGER := 24;
   constant H_SYNCTIME   : INTEGER := 72;
   constant H_BACKPORCH  : INTEGER := 128;
   constant H_PERIOD     : INTEGER := H_SYNCTIME + H_PIXELS + H_FRONTPORCH + H_BACKPORCH; -- 1024

   -- vertical timing parameters
   constant V_LINES      : INTEGER := 600;
   constant V_FRONTPORCH : INTEGER := 1;
   constant V_SYNCTIME   : INTEGER := 2;
   constant V_BACKPORCH  : INTEGER := 22;
   constant V_PERIOD     : INTEGER := V_SYNCTIME + V_LINES + V_FRONTPORCH + V_BACKPORCH; -- 625

	-- pixel
	constant TOTAL_PIXEL  : INTEGER := 480000;

end tft_pkg;
-----------------------------------------------------------------------