-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- TFTSURFER_TOP.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
-- 
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
library machxo2;
use machxo2.components.all;
-----------------------------------------------------------------------
entity tftsurfer_top is
   port(
      -- system interface
      rst_button        : in std_logic;
		tft_on_off_button : in std_logic;
      ext_osc_clk       : in std_logic;
		-- test
		-- test_pt1          : out std_logic;
		-- test_pt2          : out std_logic;
		-- test_pt3          : out std_logic;
		-- test_pt4          : out std_logic;
		-- test_pt5          : out std_logic;
		-- sdram interface
      sdram_clk         : out std_logic;
		sdram_dq          : inout std_logic_vector(15 downto 0);
		sdram_a           : out std_logic_vector(11 downto 0);
		sdram_ba          : out std_logic_vector(1 downto 0);
		sdram_cke         : out std_logic;
		sdram_csn         : out std_logic;
		sdram_rasn        : out std_logic;
		sdram_casn        : out std_logic;
		sdram_wen         : out std_logic;
		sdram_dqm         : out std_logic_vector(1 downto 0);
      -- spi flash interface
      spi_csn           : out std_logic;
      spi_clk           : out std_logic;
      spi_d             : out std_logic;
      spi_q             : in std_logic;
		-- tft and LED backlight enable
		tft_12v_bl_en     : out std_logic;
		tft_3v3_en        : out std_logic;
      -- tft interface  
		tft_clk           : out std_logic;
      tft_hsync         : out std_logic;
      tft_vsync         : out std_logic;
      tft_dtmg          : out std_logic;
      tft_red           : out std_logic_vector(5 downto 0);
      tft_green         : out std_logic_vector(5 downto 0);
      tft_blue          : out std_logic_vector(5 downto 0);
	  ----------------------------------------------------
	    WB_RST : IN std_logic;
		WB_CLK : IN std_logic;
		WB_CTI_I : IN std_logic_vector(2 downto 0);
		WB_BTE_I : IN std_logic_vector(1 downto 0);
		WB_ADR_I : IN std_logic_vector(31 downto 0);
		WB_DAT_I : IN std_logic_vector(31 downto 0);
		WB_SEL_I : IN std_logic_vector(3 downto 0);
		WB_WE_I : IN std_logic;
		WB_STB_I : IN std_logic;
		WB_CYC_I : IN std_logic;
		WB_ACK_O : OUT std_logic;
		WB_DAT_O : OUT std_logic_vector(31 downto 0);
		WB_ERR_O : OUT std_logic;
		WB_RTY_O : OUT std_logic;
		selezione_bebug : IN STD_LOGIC;
		mode8_1			 : IN std_logic;
		pageselPIO_OUT :  in std_logic_vector(7 downto 0)
   );
end tftsurfer_top;
-----------------------------------------------------------------------
architecture rtl_tftsurfer_top of tftsurfer_top is

	component oddrxe
		generic(
			gsr : string := "enabled"
		);
		port(
			d0, d1, sclk, rst : in std_logic;
			q  : out std_logic
		);
	end component;
   

	component osch
		generic(
			--nom_freq : string := "2.08"
			nom_freq : string := "10.23"
		);
		port(
			stdby    : in std_logic;
			osc      : out std_logic;
			sedstdby : out std_logic
		);
	end component;

	component pll_block
		port(
			clki   : in  std_logic;
			clkop  : out  std_logic;
			clkos  : out  std_logic;
			clkos2 : out  std_logic;
			clkos3 : out  std_logic;
			lock   : out  std_logic
		);
	end component;

	component tft_on_off_button_ctrl
		port(
			sys_rst           : in std_logic;
			osch_clk          : in std_logic;
			tft_on_off_button : in std_logic; -- pull-up, active low
			tft_on            : out std_logic; -- osch_clk domaain
			tft_off           : out std_logic; -- osch_clk domaain
			test_button       : out std_logic -- led test
		);
	end component;

	component power_manager
		port(
			sys_rst     : in std_logic;
			sys_clk     : in std_logic;
			tft_clk     : in std_logic;
			osch_clk    : in std_logic;
			pll_lock    : in std_logic;
			tft_off     : in std_logic;
			tft_on      : in std_logic;
			tft_vtick   : in std_logic;
			delay_100us : out std_logic;
			tft_vdd_en  : out std_logic;
			tft_avdd_en : out std_logic; -- SH43
			tft_disp_en : out std_logic; -- SH43
			tft_sig_en  : out std_logic;
			tft_rgb_en  : out std_logic;
			tft_led_en  : out std_logic
		);
	end component;

	component wb_interface 
    port (
		WB_RST        : IN STD_LOGIC;
		WB_CLK        : IN STD_LOGIC;
		WB_CTI_I      : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
		WB_BTE_I      : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		WB_ADR_I      : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_DAT_I      : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_SEL_I      : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
		WB_WE_I       : IN STD_LOGIC;
		WB_STB_I      : IN STD_LOGIC;
		WB_CYC_I      : IN STD_LOGIC;
		WB_ACK_O      : OUT STD_LOGIC;
		WB_DAT_O      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_ERR_O      : OUT STD_LOGIC;
		WB_RTY_O      : OUT STD_LOGIC;
		-- interfacce verso fifo per la lettura
		R_wb_add_fifo_Data 	: OUT STD_LOGIC_VECTOR(21 DOWNTO 0);
		R_wb_add_fifo_WrEn 	: OUT STD_LOGIC; 
		R_wb_data_fifo_RdEn	: OUT STD_LOGIC;  
		R_wb_data_fifo_Q		: IN STD_LOGIC_VECTOR(15 DOWNTO 0);  
		R_wb_data_fifo_Empty	: IN STD_LOGIC; 
		-- interfaccia verso fifo di scrittura
		W_fifo_wr_req_sig : OUT STD_LOGIC;  
		W_fifo_wr_add_sig : OUT STD_LOGIC_VECTOR(21 DOWNTO 0);
		W_fifo_wr_rgb_sig : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		W_fifo_full		  : IN STD_LOGIC 
			);
	end component;

	component tftsurfer_demo
		port(
			-- system interface
			sys_rst         : in std_logic;
			tft_clk         : in std_logic;
			demo_start      : in std_logic;
			-- test point x debug
			test_pt         : out std_logic;
			-- tft_timing_ctrl interface
			tft_bank        : out std_logic_vector(1 downto 0);
			tft_sect        : out std_logic_vector(2 downto 0);
			-- pixel_addr_data_fifo interface
			fifo_wr_req     : out std_logic;
			fifo_wr_add     : out std_logic_vector(21 downto 0);
			fifo_wr_rgb     : out std_logic_vector(15 downto 0);
			-- spi flash interface
			spi_csn         : out std_logic;
			spi_clk         : out std_logic;
			spi_d           : out std_logic;
			spi_q           : in std_logic
		);
	end component;

	component pixel_addr_data_fifo
		port(
			-- sys interface
			sys_rst     : in std_logic;
			sys_clk     : in std_logic; -- system clock for READ (SDRAM clk)
			sys_div_clk : in std_logic; -- WRITE clock (uC interface)
			-- WRITE interface
			fifo_wr_req : in std_logic;
			fifo_wr_add : in std_logic_vector(21 downto 0);
			fifo_wr_rgb : in std_logic_vector(15 downto 0);
			fifo_full   : out std_logic;
			-- READ interface
			fifo_rd_req : in std_logic;
			fifo_rd_add : out std_logic_vector(21 downto 0);
			fifo_rd_rgb : out std_logic_vector(15 downto 0);
			fifo_rd_cnt : out std_logic_vector(4 downto 0);
			fifo_empty  : out std_logic
		);
	end component;

	component monster_manager
		port(
			-- system interface
			sys_rst          : in std_logic;
			sys_clk          : in std_logic;
			-- power_ctrl interface
			delay_100us      : in std_logic;
			-- SDRAM/monster_manager ready for read and write
			sys_sdram_ready_rw : out std_logic;
			-- sdram_ctrl interface
			sys_sdram_init   : out std_logic;
			sys_sdram_ready  : in std_logic;
			sys_r_wn         : out std_logic;
			sys_adsn         : out std_logic;
			-- system address definition
			-- bank(21-20) -> 4
			-- row(19-8)   -> 4096
			-- column(7-0) -> 256
			sys_addr         : out std_logic_vector(21 downto 0);
			sys_be           : out std_logic_vector(1 downto 0);
			-- write data
			sys_data_in      : out std_logic_vector(15 downto 0);
			-- read data and data valid
			sys_data_out     : in std_logic_vector(15 downto 0);
			sys_data_val     : in std_logic;
			-- read or write cycle is completed
			sys_cycle_end    : in std_logic;
			-- auto refresh
			sys_ref_req      : out std_logic;
			sys_ref_ack      : in std_logic;
			-- tft_timing_ctrl interface
			tft_row_start    : in std_logic;
			tft_sdram_dtmg   : in std_logic;
			tft_sdram_addr   : in std_logic_vector(21 downto 0);
			tft_sdram_rgb_en : out std_logic;
			tft_sdram_red    : out std_logic_vector(4 downto 0);
			tft_sdram_green  : out std_logic_vector(5 downto 0);
			tft_sdram_blue   : out std_logic_vector(4 downto 0);
			-- data_addr_fifo interface
			fifo_rd_req      : out std_logic;
			fifo_rd_add      : in std_logic_vector(21 downto 0); -- 22 bit
			fifo_rd_rgb      : in std_logic_vector(15 downto 0); -- 16 bit RGB 565
			fifo_rd_cnt      : in std_logic_vector(4 downto 0);
			fifo_rd_empty    : in std_logic;
			mode8_1          : in std_logic; -- selezione il modo di accesso
			-- (porta per permettere le letture)
			--fifo address
			wb_add_req       : out std_logic;
			wb_add           : in std_logic_vector(21 downto 0); -- 22 bit
			wb_add_empty     : in std_logic;
			--fifo data
			wb_data_we       : out std_logic;
			wb_data          : out std_logic_vector(15 downto 0)
		);
	end component;

	component sdram_ctrl
		port(
			-- system interface
			sys_clk         : in std_logic;
			sys_rst         : in std_logic;
			sys_sdram_init  : in std_logic;
			sys_sdram_ready : out std_logic;
			sys_r_wn        : in std_logic;
			sys_adsn        : in std_logic;
			-- MICRON 64Mbit: system address definition
			-- bank(21-20) -> 4
			-- row(19-8)   -> 4096
			-- column(7-0) -> 256
			sys_addr        : in std_logic_vector(21 downto 0);
			sys_be          : in std_logic_vector(1 downto 0); -- active low (when high data is masked
			-- write data
			sys_data_in     : in std_logic_vector(15 downto 0);
			-- read data and data valid
			sys_data_out    : out std_logic_vector(15 downto 0);
			sys_data_val    : out std_logic;
			-- read or write cycle is completed
			sys_cycle_end   : out std_logic;
			-- auto refresh
			sys_ref_req     : in std_logic;
			sys_ref_ack     : out std_logic;
			-- sdram interface
			sdram_dq        : inout std_logic_vector(15 downto 0);
			sdram_a         : out std_logic_vector(11 downto 0);
			sdram_ba        : out std_logic_vector(1 downto 0);
			sdram_cke       : out std_logic;
			sdram_csn       : out std_logic;
			sdram_rasn      : out std_logic;
			sdram_casn      : out std_logic;
			sdram_wen       : out std_logic;
			sdram_dqm       : out std_logic_vector(1 downto 0)
		);
	end component;

	component tft_pixel_rgb_pipeliner
		port(
			-- sys interface
			sys_rst             : in std_logic;
			sys_clk             : in std_logic; -- WRITE (monster_manager)
			tft_clk             : in std_logic; -- READ (tft_timing_ctrl)
			-- WRITE: monster_manager interface
			tft_sdram_rgb_en    : in std_logic;
			tft_sdram_red_in    : in std_logic_vector(4 downto 0); -- 5 bit
			tft_sdram_green_in  : in std_logic_vector(5 downto 0); -- 6 bit
			tft_sdram_blue_in   : in std_logic_vector(4 downto 0); -- 5 bit
			-- READ: tft_timing_ctrl interface
			tft_sdram_red_out   : out std_logic_vector(4 downto 0); -- 5 bit
			tft_sdram_green_out : out std_logic_vector(5 downto 0); -- 6 bit
			tft_sdram_blue_out  : out std_logic_vector(4 downto 0)  -- 5 bit
		);
	end component;

	component tft_timing_ctrl
		port(
			-- system interface
			sys_rst         : in std_logic;
			sys_clk         : in std_logic;
			tft_clk         : in std_logic;
			-- microcontroller interface
			sdram_test      : in std_logic;
			tft_test        : in std_logic;
			tft_bank        : in std_logic_vector(1 downto 0);
			tft_sect        : in std_logic_vector(2 downto 0);
			-- power manager interface
			tft_sig_en      : in std_logic;
			tft_rgb_en      : in std_logic;
			tft_vtick       : out std_logic;
			-- monster manager interface
			tft_row_start   : out std_logic;
			tft_sdram_dtmg  : out std_logic;
			tft_sdram_addr  : out std_logic_vector(21 downto 0);
			tft_sdram_red   : in std_logic_vector(4 downto 0);
			tft_sdram_green : in std_logic_vector(5 downto 0);
			tft_sdram_blue  : in std_logic_vector(4 downto 0);
			-- tft interface
			tft_hsync       : out std_logic;
			tft_vsync       : out std_logic;
			tft_dtmg        : out std_logic;
			tft_red         : out std_logic_vector(7 downto 0);
			tft_green       : out std_logic_vector(7 downto 0);
			tft_blue        : out std_logic_vector(7 downto 0)
		);
	end component;

	component tx_lvds_71
		port(
			clk      : in std_logic;
			clk_s    : in std_logic; 
			clkout   : out std_logic;
			reset    : in std_logic; 
			sclk     : out std_logic;
			tx_ready : out std_logic; 
			dataout0 : in std_logic_vector(6 downto 0); 
			dataout1 : in std_logic_vector(6 downto 0); 
			dataout2 : in std_logic_vector(6 downto 0); 
			dataout3 : in std_logic_vector(6 downto 0); 
			dout     : out std_logic_vector(3 downto 0)
		);
	end component;
	
	component wb_add_fifo
        port (Data : in std_logic_vector(21 downto 0); 
        WrClock: in std_logic; RdClock: in std_logic; WrEn: in std_logic; 
        RdEn: in std_logic; Reset: in std_logic; RPReset: in std_logic; 
        Q : out std_logic_vector(21 downto 0); Empty: out std_logic; 
        Full: out std_logic
    );
    end component;
	component wb_data_fifo
    port (Data: in  std_logic_vector(15 downto 0); 
        WrClock: in  std_logic; RdClock: in  std_logic; 
        WrEn: in  std_logic; RdEn: in  std_logic; Reset: in  std_logic; 
        RPReset: in  std_logic; Q: out  std_logic_vector(15 downto 0); 
        Empty: out  std_logic; Full: out  std_logic);
end component;

	signal VDD : std_logic;
	signal GND : std_logic;

	signal osch_clk_sig : std_logic;

	signal sys_rst_sig     : std_logic;
	signal sys_clk_sig     : std_logic;
	signal sys_clk_180_sig : std_logic;
	signal sys_div_clk_sig : std_logic;
	signal tft_clk_sig     : std_logic;
	signal pll_lock_sig    : std_logic;

	signal tft_on_sig  : std_logic;
	signal tft_off_sig : std_logic;

	signal tft_vtick_sig   : std_logic;
	signal delay_100us_sig : std_logic;
	signal tft_vdd_en_sig  : std_logic;
	signal tft_avdd_en_sig : std_logic;
	signal tft_disp_en_sig : std_logic;
	signal tft_sig_en_sig  : std_logic;
	signal tft_rgb_en_sig  : std_logic;
	signal tft_led_en_sig  : std_logic;

	signal sys_sdram_ready_rw_sig : std_logic;

	signal sys_sdram_ready_sig : std_logic;

	signal fifo_wr_req_sig : std_logic;
	signal fifo_wr_add_sig : std_logic_vector(21 downto 0);
	signal fifo_wr_rgb_sig : std_logic_vector(15 downto 0);
	signal fifo_full_sig   : std_logic;

	signal fifo_rd_req_sig : std_logic;
	signal fifo_rd_add_sig : std_logic_vector(21 downto 0);
	signal fifo_rd_rgb_sig : std_logic_vector(15 downto 0);
	signal fifo_rd_cnt_sig : std_logic_vector(4 downto 0);
	signal fifo_empty_sig  : std_logic;

	signal sys_sdram_init_sig : std_logic;

	signal sys_r_wn_sig : std_logic;
	signal sys_adsn_sig : std_logic;

	signal sys_addr_sig      : std_logic_vector(21 downto 0);
	signal sys_be_sig        : std_logic_vector(1 downto 0);
	signal sys_data_in_sig   : std_logic_vector(15 downto 0);
	signal sys_data_out_sig  : std_logic_vector(15 downto 0);
	signal sys_data_val_sig  : std_logic;
	signal sys_cycle_end_sig : std_logic;
	signal sys_ref_req_sig   : std_logic;
	signal sys_ref_ack_sig   : std_logic;

	signal tft_row_start_sig   : std_logic;
	signal tft_sdram_dtmg_sig  : std_logic;
	signal tft_sdram_addr_sig  : std_logic_vector(21 downto 0);
	
	signal tft_sdram_rgb_en_sig : std_logic;

	signal tft_sdram_red_in_sig   : std_logic_vector(4 downto 0);
	signal tft_sdram_green_in_sig : std_logic_vector(5 downto 0);
	signal tft_sdram_blue_in_sig  : std_logic_vector(4 downto 0);

	signal tft_sdram_red_out_sig   : std_logic_vector(4 downto 0);
	signal tft_sdram_green_out_sig : std_logic_vector(5 downto 0);
	signal tft_sdram_blue_out_sig  : std_logic_vector(4 downto 0);

	signal sdram_test_sig : std_logic;
	signal tft_test_sig   : std_logic;
	signal tft_bank_sig   : std_logic_vector(1 downto 0);
	signal tft_sect_sig   : std_logic_vector(2 downto 0);

   signal tft_hsync_sig : std_logic;
   signal tft_vsync_sig : std_logic;
   signal tft_dtmg_sig  : std_logic;

   signal tft_red_sig   : std_logic_vector(7 downto 0);
   signal tft_green_sig : std_logic_vector(7 downto 0);
   signal tft_blue_sig  : std_logic_vector(7 downto 0);

   signal tft_clk_out_sig : std_logic;
   signal sclk_sig     : std_logic;
   signal tx_ready_sig : std_logic;

   signal lvds_71_data0_sig : std_logic_vector(6 downto 0);
   signal lvds_71_data1_sig : std_logic_vector(6 downto 0);
   signal lvds_71_data2_sig : std_logic_vector(6 downto 0);
   signal lvds_71_data3_sig : std_logic_vector(6 downto 0);

   signal tft_data_sig : std_logic_vector(3 downto 0);
	
	signal osch_clk_cnt : std_logic_vector(7 downto 0);

	signal led_pwm_clk_sig    : std_logic;

	signal led_pwm_cnt    : std_logic_vector(6 downto 0);
	signal buzzer_pwm_cnt : std_logic_vector(6 downto 0);

	signal tft_dimming : std_logic;

	------------------------------------------------------------------------------
	signal wb_add_req_sig       : std_logic;
	signal wb_add_sig           : std_logic_vector(21 downto 0); -- 22 bit
	signal wb_add_empty_sig     : std_logic;
	signal wb_data_we_sig       : std_logic;
	signal wb_data_sig          : std_logic_vector(15 downto 0);
	------------------------------------------------------------------------------
	
	signal	R_wb_add_fifo_Data 		:  STD_LOGIC_VECTOR(21 DOWNTO 0);
	signal	R_wb_add_fifo_WrEn 		:  STD_LOGIC; 
	signal	R_wb_data_fifo_RdEn		:  STD_LOGIC;  
	signal	R_wb_data_fifo_Q		:  STD_LOGIC_VECTOR(15 DOWNTO 0);  
	signal	R_wb_data_fifo_Empty	:  STD_LOGIC; 
	
	signal	W_fifo_wr_req_sig :  STD_LOGIC;  
	signal	W_fifo_wr_add_sig :  STD_LOGIC_VECTOR(21 DOWNTO 0);
	signal	W_fifo_wr_rgb_sig :  STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal	W_fifo_full		  :  STD_LOGIC; 
	
	signal WB_fifo_rd_req_sig : std_logic;
	signal WB_fifo_rd_add_sig : std_logic_vector(21 downto 0);
	signal WB_fifo_rd_rgb_sig : std_logic_vector(15 downto 0);
	signal WB_fifo_rd_cnt_sig : std_logic_vector(4 downto 0);
	signal WB_fifo_empty_sig  : std_logic;
	
	signal ALL_fifo_rd_req_sig : std_logic;
	signal ALL_fifo_rd_add_sig : std_logic_vector(21 downto 0);
	signal ALL_fifo_rd_rgb_sig : std_logic_vector(15 downto 0);
	signal ALL_fifo_rd_cnt_sig : std_logic_vector(4 downto 0);
	signal ALL_fifo_empty_sig  : std_logic;
           
begin

	VDD <= '1';
	GND <= '0';
	
	sdram_cke <= VDD;

	sys_rst_sig <= not rst_button;

	OSCH_INST: osch
		generic map(
--			nom_freq => "2.08"
			nom_freq => "10.23"
		)
		port map(
			stdby    => GND,
			osc      => osch_clk_sig,
			sedstdby => open
		);

	PLL_BLOCK_INST: pll_block
		port map(
			clki   => ext_osc_clk,
--			clki   => osch_clk_sig,
			clkop  => sys_clk_sig,
			clkos  => sys_clk_180_sig,
			clkos2 => sys_div_clk_sig,
			clkos3 => tft_clk_sig,
			lock   => pll_lock_sig
		);
	
   ------------------------
	-- functional simulation
   ------------------------
   --sdram_clk <= sys_clk_sig;

   -----------------------
	-- final implementation
   -----------------------
   sdram_clk <= not sys_clk_sig;
	-- ODDRXE_INST: oddrxe
		-- port map(
			-- d0  => GND,
			-- d1  => VDD,
			-- sclk => sys_clk_sig,
			-- rst  => sys_rst_sig,
			-- q    => sdram_clk
		-- );

	TFT_ON_OFF_BUTTON_CTRL_INST: tft_on_off_button_ctrl
		port map(
			sys_rst           => sys_rst_sig,
			osch_clk          => osch_clk_sig,
			tft_on_off_button => tft_on_off_button,
			tft_on            => tft_on_sig,
			tft_off           => tft_off_sig,
			test_button       => open
		);

	POWER_MANAGER_INST: power_manager
		port map(
			sys_rst     => sys_rst_sig,
			sys_clk     => sys_clk_sig,
			tft_clk     => tft_clk_sig,
			osch_clk    => osch_clk_sig,
			pll_lock    => pll_lock_sig,
			tft_off     => tft_off_sig,
			tft_on      => tft_on_sig,
			tft_vtick   => tft_vtick_sig,
			delay_100us => delay_100us_sig,
			tft_vdd_en  => tft_vdd_en_sig,
			tft_avdd_en => tft_avdd_en_sig,
			tft_disp_en => tft_disp_en_sig,
			tft_sig_en  => tft_sig_en_sig,
			tft_rgb_en  => tft_rgb_en_sig,
			tft_led_en  => tft_led_en_sig
		);

	-- SDRAM_TEST_GENERATOR_INST: sdram_test_generator
		-- port map(
			-- -- system interface
			-- sys_rst           => sys_rst_sig,
			-- tft_clk           => tft_clk_sig,
			-- -- monster_manager interface
			-- sdram_ready_rw    => sys_sdram_ready_rw_sig,
			-- -- sdram fifo write interface
			-- sdram_fifo_wr_req => fifo_wr_req_sig,
			-- sdram_fifo_wr_add => fifo_wr_add_sig,
			-- sdram_fifo_wr_rgb => fifo_wr_rgb_sig
		-- );

	TFTSURFER_DEMO_INST: tftsurfer_demo
		port map(
			-- system interface
			sys_rst         => sys_rst_sig,
			tft_clk         => tft_clk_sig,
			demo_start      => sys_sdram_ready_rw_sig,  --FABRIZIO: questo fa partire la copia da flash a SDRAM
			-- test point x debug
			test_pt         => open,
			-- tft_timing_ctrl interface
			tft_bank        => open , --tft_bank_sig,
			tft_sect        => open , --tft_sect_sig,
			-- pixel_addr_data_fifo interface
			fifo_wr_req     => fifo_wr_req_sig,
			fifo_wr_add     => fifo_wr_add_sig,
			fifo_wr_rgb     => fifo_wr_rgb_sig,
			-- spi flash interface
			spi_csn         => spi_csn,
			spi_clk         => spi_clk,
			spi_d           => spi_d,
			spi_q           => spi_q
		);

	PIXEL_ADDR_DATA_FIFO_INST: pixel_addr_data_fifo
		port map(
			-- sys interface
			sys_rst     => sys_rst_sig,
			sys_clk     => sys_clk_sig,
			sys_div_clk => tft_clk_sig,
			-- WRITE interface
			fifo_wr_req => fifo_wr_req_sig,  
			fifo_wr_add => fifo_wr_add_sig,
			fifo_wr_rgb => fifo_wr_rgb_sig,
			fifo_full   => fifo_full_sig,
			-- READ interface
			fifo_rd_req => fifo_rd_req_sig,
			fifo_rd_add => fifo_rd_add_sig,
			fifo_rd_rgb => fifo_rd_rgb_sig,
			fifo_rd_cnt => fifo_rd_cnt_sig,
			fifo_empty  => fifo_empty_sig
		);

	MONSTER_MANAGER_INST: monster_manager
		port map(
			-- system interface
			sys_rst          => sys_rst_sig,
			sys_clk          => sys_clk_sig,
			-- power_ctrl interface
			delay_100us      => delay_100us_sig,
			-- SDRAM/monster_manager ready for read and write
			sys_sdram_ready_rw => sys_sdram_ready_rw_sig,
			-- sdram_ctrl interface
			sys_sdram_init   => sys_sdram_init_sig,
			sys_sdram_ready  => sys_sdram_ready_sig,
			sys_r_wn         => sys_r_wn_sig,
			sys_adsn         => sys_adsn_sig,
			-- system address definition
			-- bank(21-20) -> 4
			-- row(19-8)   -> 4096
			-- column(7-0) -> 256
			sys_addr         => sys_addr_sig,
			sys_be           => sys_be_sig,
			-- write data
			sys_data_in      => sys_data_in_sig,
			-- read data and data valid
			sys_data_out     => sys_data_out_sig,
			sys_data_val     => sys_data_val_sig,
			-- read or write cycle is completed
			sys_cycle_end    => sys_cycle_end_sig,
			-- auto refresh
			sys_ref_req      => sys_ref_req_sig,
			sys_ref_ack      => sys_ref_ack_sig,
			-- tft_timing_ctrl interface
			tft_row_start    => tft_row_start_sig,
			tft_sdram_dtmg   => tft_sdram_dtmg_sig,
			tft_sdram_addr   => tft_sdram_addr_sig,
			tft_sdram_rgb_en => tft_sdram_rgb_en_sig,
			tft_sdram_red    => tft_sdram_red_in_sig,
			tft_sdram_green  => tft_sdram_green_in_sig,
			tft_sdram_blue   => tft_sdram_blue_in_sig,
			-- data_addr_fifo interface
			fifo_rd_req      => ALL_fifo_rd_req_sig,
			fifo_rd_add      => ALL_fifo_rd_add_sig,
			fifo_rd_rgb      => ALL_fifo_rd_rgb_sig,
			fifo_rd_cnt      => ALL_fifo_rd_cnt_sig,
			fifo_rd_empty    => ALL_fifo_empty_sig,
			mode8_1          => mode8_1, -- selezione il modo di accesso
			-- (porta per permettere le letture)
			--fifo address
			wb_add_req       => wb_add_req_sig,
			wb_add           => wb_add_sig,
			wb_add_empty     => wb_add_empty_sig,
			--fifo data
			wb_data_we       => wb_data_we_sig,
			wb_data          => wb_data_sig
		);


inst_wb_add_fifo : wb_add_fifo
    port map (	Reset				=>sys_rst_sig, 
				RPReset				=>sys_rst_sig, 
				Data(21 downto 0)	=>R_wb_add_fifo_Data, 
				WrClock				=>WB_CLK, 
				WrEn				=>R_wb_add_fifo_WrEn, 
				RdClock				=>sys_clk_sig, 
				RdEn				=>wb_add_req_sig, 
				Q(21 downto 0)		=>wb_add_sig, 
				Empty				=>wb_add_empty_sig, 
				Full				=>open);

inst_wb_data_fifo : wb_data_fifo
    port map (	Reset				=>sys_rst_sig, 
				RPReset				=>sys_rst_sig, 
				Data(15 downto 0)	=>wb_data_sig, 
				WrClock				=> sys_clk_sig, 
				WrEn				=>wb_data_we_sig, 
				
				RdClock				=>WB_CLK, 
				RdEn				=>R_wb_data_fifo_RdEn,  
				Q(15 downto 0)		=>R_wb_data_fifo_Q, 
				Empty				=>R_wb_data_fifo_Empty, 
				Full				=>open);
		
			
	inst_wb_interface : wb_interface 
    port map (
		WB_RST        =>  sys_rst_sig ,   
		WB_CLK        =>  WB_CLK      ,
		WB_CTI_I      =>  WB_CTI_I    ,
		WB_BTE_I      =>  WB_BTE_I    ,
		WB_ADR_I      =>  WB_ADR_I    ,
		WB_DAT_I      =>  WB_DAT_I    ,
		WB_SEL_I      =>  WB_SEL_I    ,
		WB_WE_I       =>  WB_WE_I     ,
		WB_STB_I      =>  WB_STB_I    ,
		WB_CYC_I      =>  WB_CYC_I    ,
		WB_ACK_O      =>  WB_ACK_O    ,
		WB_DAT_O      =>  WB_DAT_O    ,
		WB_ERR_O      =>  WB_ERR_O    ,
		WB_RTY_O      =>  WB_RTY_O    ,
		-- interfacce verso fifo per la lettura
		R_wb_add_fifo_Data 		=>  R_wb_add_fifo_Data 	  ,
		R_wb_add_fifo_WrEn 		=>  R_wb_add_fifo_WrEn 	  ,
		
		R_wb_data_fifo_RdEn		=>  R_wb_data_fifo_RdEn	  ,
		R_wb_data_fifo_Q		=>  R_wb_data_fifo_Q	  ,
		R_wb_data_fifo_Empty	=>  R_wb_data_fifo_Empty  ,
		-- interfaccia verso fifo di scrittura (FATTO)
		W_fifo_wr_req_sig       =>  W_fifo_wr_req_sig   ,
		W_fifo_wr_add_sig       =>  W_fifo_wr_add_sig   ,
		W_fifo_wr_rgb_sig       =>  W_fifo_wr_rgb_sig   ,
		W_fifo_full		        =>  W_fifo_full		   
			);

			
			
			
			
			
		scrittura_da_wishbone_pixel_addr_data_fifo : pixel_addr_data_fifo
		port map(
			-- sys interface
			sys_rst     => sys_rst_sig,
			sys_clk     => sys_clk_sig,
			sys_div_clk => WB_CLK,
			-- WRITE interface
			fifo_wr_req => W_fifo_wr_req_sig,
			fifo_wr_add => W_fifo_wr_add_sig,
			fifo_wr_rgb => W_fifo_wr_rgb_sig,
			fifo_full   => W_fifo_full,
			-- READ interface
			fifo_rd_req => WB_fifo_rd_req_sig,
			fifo_rd_add => WB_fifo_rd_add_sig,
			fifo_rd_rgb => WB_fifo_rd_rgb_sig,
			fifo_rd_cnt => WB_fifo_rd_cnt_sig,
			fifo_empty  => WB_fifo_empty_sig
		);	


	 WB_fifo_rd_req_sig    	<= ALL_fifo_rd_req_sig when (selezione_bebug='0')  else '0';
	 fifo_rd_req_sig   		<= ALL_fifo_rd_req_sig when (selezione_bebug='1')  else '0';	
	 
     ALL_fifo_rd_add_sig	 <= WB_fifo_rd_add_sig when (selezione_bebug='0')  else  fifo_rd_add_sig;
     ALL_fifo_rd_rgb_sig	 <= WB_fifo_rd_rgb_sig when (selezione_bebug='0')  else  fifo_rd_rgb_sig;
     ALL_fifo_rd_cnt_sig	 <= WB_fifo_rd_cnt_sig when (selezione_bebug='0')  else  fifo_rd_cnt_sig;
     ALL_fifo_empty_sig 	 <= WB_fifo_empty_sig  when (selezione_bebug='0')  else  fifo_empty_sig ;
	
	SDRAM_CTRL_INST: sdram_ctrl
		port map(
			-- system interface
			sys_rst         => sys_rst_sig,
			sys_clk         => sys_clk_sig,
			sys_sdram_init  => sys_sdram_init_sig,
			sys_sdram_ready => sys_sdram_ready_sig,
			sys_r_wn        => sys_r_wn_sig,
			sys_adsn        => sys_adsn_sig,
			-- MICRON 64Mbit: system address definition
			-- bank(21-20) -> 4
			-- row(19-8)   -> 4096
			-- column(7-0) -> 256
			sys_addr        => sys_addr_sig,
			sys_be          => sys_be_sig,
			-- write data
			sys_data_in     => sys_data_in_sig,
			-- read data and data valid
			sys_data_out    => sys_data_out_sig,
			sys_data_val    => sys_data_val_sig,
			-- read or write cycle is completed
			sys_cycle_end   => sys_cycle_end_sig,
			-- auto refresh
			sys_ref_req     => sys_ref_req_sig,
			sys_ref_ack     => sys_ref_ack_sig,
			-- sdram interface
			sdram_dq        => sdram_dq,
			sdram_a         => sdram_a,
			sdram_ba        => sdram_ba,
			sdram_cke       => open,
			sdram_csn       => sdram_csn,
			sdram_rasn      => sdram_rasn,
			sdram_casn      => sdram_casn,
			sdram_wen       => sdram_wen,
			sdram_dqm       => sdram_dqm 
		);

	TFT_PIXEL_RGB_PIPELINER_INST: tft_pixel_rgb_pipeliner
		port map(
			-- sys interface
			sys_rst             => sys_rst_sig,
			sys_clk             => sys_clk_sig,
			tft_clk             => tft_clk_sig,
			-- WRITE: monster_manager interface
			tft_sdram_rgb_en    => tft_sdram_rgb_en_sig,
			tft_sdram_red_in    => tft_sdram_red_in_sig,
			tft_sdram_green_in  => tft_sdram_green_in_sig,
			tft_sdram_blue_in   => tft_sdram_blue_in_sig,
			-- READ: tft_timing_ctrl interface
			tft_sdram_red_out   => tft_sdram_red_out_sig,
			tft_sdram_green_out => tft_sdram_green_out_sig,
			tft_sdram_blue_out  => tft_sdram_blue_out_sig
		);

	-- SDRAM test
	sdram_test_sig <= '0';
	tft_test_sig   <= '1';

	tft_bank_sig <= pageselPIO_OUT(2 downto 1);
	tft_sect_sig <= "00" & pageselPIO_OUT(0);
	
	TFT_TIMING_CTRL_INST: tft_timing_ctrl
		port map(
			-- system interface
			sys_rst         => sys_rst_sig,
			sys_clk         => sys_clk_sig,
			tft_clk         => tft_clk_sig,
			-- microcontroller interface
			sdram_test      => sdram_test_sig,
			tft_test        => tft_test_sig,
			tft_bank        => tft_bank_sig,
			tft_sect        => tft_sect_sig,
			-- power manager interface
			tft_sig_en      => tft_sig_en_sig,
			tft_rgb_en      => tft_rgb_en_sig,
			tft_vtick       => tft_vtick_sig,
			-- monster manager interface
			tft_row_start   => tft_row_start_sig,
			tft_sdram_dtmg  => tft_sdram_dtmg_sig,
			tft_sdram_addr  => tft_sdram_addr_sig,
			tft_sdram_red   => tft_sdram_red_out_sig,
			tft_sdram_green => tft_sdram_green_out_sig,
			tft_sdram_blue  => tft_sdram_blue_out_sig,
			-- tft interface
			tft_hsync       => tft_hsync_sig,
			tft_vsync       => tft_vsync_sig,
			tft_dtmg        => tft_dtmg_sig,
			tft_red         => tft_red_sig,
			tft_green       => tft_green_sig,
			tft_blue        => tft_blue_sig
		);

	tft_clk   <= tft_clk_sig;
	tft_hsync <= tft_hsync_sig;
	tft_vsync <= tft_vsync_sig;
	tft_dtmg  <= tft_dtmg_sig;

	tft_red   <= tft_red_sig(7 downto 2);
	tft_green <= tft_green_sig(7 downto 2);
	tft_blue  <= tft_blue_sig(7 downto 2);
	-- tft_red   <= ('1' & tft_red_sig(7 downto 3));
	-- tft_green <= ('1' & tft_green_sig(7 downto 3));
	-- tft_blue  <= ('1' & tft_blue_sig(7 downto 3));

	tft_12v_bl_en <= GND;
	tft_3v3_en    <= GND;

   -------------------------
   -- OSCH_CLK_PROC process
   -------------------------
   OSCH_CLK_PROC: process(osch_clk_sig)
   begin
      if (osch_clk_sig = '1' and osch_clk_sig'event) then
         osch_clk_cnt <= osch_clk_cnt + 1;
      end if;
   end process OSCH_CLK_PROC;

	-- LED PWM clock signal
	led_pwm_clk_sig <= osch_clk_cnt(6);

   -------------------------
   -- LED_PWM_CNT_PROC process
   -------------------------
   LED_PWM_CNT_PROC: process(led_pwm_clk_sig)
   begin
      if (led_pwm_clk_sig = '1' and led_pwm_clk_sig'event) then
         led_pwm_cnt <= led_pwm_cnt + 1;
      end if;
   end process LED_PWM_CNT_PROC;

   ---------------------------
   -- LED_PWM_OUT_PROC process
   ---------------------------
   LED_PWM_OUT_PROC: process(sys_rst_sig, led_pwm_clk_sig)
   begin
		if (sys_rst_sig = '1') then
			tft_dimming <= '0';
      elsif (led_pwm_clk_sig = '1' and led_pwm_clk_sig'event) then
--			if ( (led_pwm_on_off_duty(6 downto 0) >= led_pwm_cnt) and (led_pwm_on_off_duty(7)= '1') ) then
			if (led_pwm_cnt <= "1100000") then
				tft_dimming <= '1';
			else
				tft_dimming <= '0';
			end if;
      end if;
   end process LED_PWM_OUT_PROC;

end rtl_tftsurfer_top;
-----------------------------------------------------------------------