-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity wb_interface is
    port (
		WB_RST        : IN STD_LOGIC;
		WB_CLK        : IN STD_LOGIC;
		WB_CTI_I      : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
		WB_BTE_I      : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		WB_ADR_I      : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_DAT_I      : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_SEL_I      : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
		WB_WE_I       : IN STD_LOGIC;
		WB_STB_I      : IN STD_LOGIC;
		WB_CYC_I      : IN STD_LOGIC;
		WB_ACK_O      : OUT STD_LOGIC;
		WB_DAT_O      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
		WB_ERR_O      : OUT STD_LOGIC;
		WB_RTY_O      : OUT STD_LOGIC;
		-- interfacce verso fifo per la lettura
		R_wb_add_fifo_Data 	: OUT STD_LOGIC_VECTOR(21 DOWNTO 0);
		R_wb_add_fifo_WrEn 	: OUT STD_LOGIC; 
		R_wb_data_fifo_RdEn	: OUT STD_LOGIC;  
		R_wb_data_fifo_Q		: IN STD_LOGIC_VECTOR(15 DOWNTO 0);  
		R_wb_data_fifo_Empty	: IN STD_LOGIC; 
		-- interfaccia verso fifo di scrittura
		W_fifo_wr_req_sig : OUT STD_LOGIC;  
		W_fifo_wr_add_sig : OUT STD_LOGIC_VECTOR(21 DOWNTO 0);
		W_fifo_wr_rgb_sig : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		W_fifo_full		  : IN STD_LOGIC 
			);
	end wb_interface;
	
architecture rtl_wb_interface of wb_interface is
   type fsm_state is (IDLE, READ1, READ2, READ3, READ4, READ5, READ6, READ7, WRITE1, WRITE2, WRITE3, WRITE4, WRITE5, WRITE6, WRITE7 );

   signal stato : fsm_state;
	signal R_wb_data_fifo_Empty_smp : std_logic;
	signal W_fifo_full_smp : std_logic;
	signal registri : std_logic;
   
begin
WB_ERR_O      <= '0' ;
WB_RTY_O      <= '0' ;

process(WB_RST, WB_CLK)
begin
if WB_RST = '1' then
		R_wb_add_fifo_Data 		<= "0000000000000000000000";
		R_wb_add_fifo_WrEn 		<= '0';
		R_wb_data_fifo_RdEn		<= '0'; 
		W_fifo_wr_req_sig 		<= '0';  
		W_fifo_wr_add_sig 		<= "0000000000000000000000";
		W_fifo_wr_rgb_sig 		<= "0000000000000000";
		WB_ACK_O      <= '0';
		WB_DAT_O      <= (others => '0');
		
		stato 					<= IDLE;
elsif WB_CLK'event and WB_CLK='1' then
	R_wb_data_fifo_Empty_smp <= R_wb_data_fifo_Empty;
	W_fifo_full_smp <= W_fifo_full;
	case STATO is
	when IDLE =>
		WB_ACK_O      <= '0';
		if 		WB_CYC_I = '1' and WB_STB_I = '1' and WB_WE_I = '1' then
			stato <= WRITE1;
		elsif   WB_CYC_I = '1' and WB_STB_I = '1' and WB_WE_I = '0' then
			stato <= READ1;
		end if;
	when READ1 =>
		R_wb_add_fifo_Data <= WB_ADR_I(21 downto 0);
		R_wb_add_fifo_WrEn <= '1';
		stato <= READ2;
	when READ2 =>
		R_wb_add_fifo_WrEn <= '0';
		if R_wb_data_fifo_Empty_smp = '0' then
			stato <= READ3;
			R_wb_data_fifo_RdEn <= '1';
		end if;
	when READ3 =>
		R_wb_data_fifo_RdEn <= '0';
		stato <= READ4;
	when READ4 =>
			WB_DAT_O <= R_wb_data_fifo_Q & R_wb_data_fifo_Q;
		WB_ACK_O      <= '1';
		if 	WB_CYC_I = '0' and WB_STB_I = '0'  then
			stato <= IDLE;
			WB_ACK_O      <= '0';
		end if;
	when WRITE1 =>
		W_fifo_wr_add_sig <= WB_ADR_I(21 downto 0);
		--if 
		W_fifo_wr_rgb_sig <= WB_DAT_I(15 downto 0);
		if W_fifo_full_smp = '0' then
			W_fifo_wr_req_sig <= '1';
			stato <= WRITE2;
			WB_ACK_O      <= '1';
		end if;
	when WRITE2 =>	
		W_fifo_wr_req_sig <= '0';
		if 	WB_CYC_I = '0' and WB_STB_I = '0'  then
			stato <= IDLE;
			WB_ACK_O      <= '0';
		end if;
	when others =>  stato <= IDLE;
	end case;
end if;
end process;

end rtl_wb_interface;