
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity SPI_top_RW is
   port(
      -- system interface
		rst        : in std_logic;
		clk : in std_logic;
     
      u_spi_csn           : in std_logic;
      u_spi_clk           : in std_logic;
      u_spi_d             : in std_logic;
      u_spi_q             : out std_logic;

	    WB_RST : IN std_logic;
		WB_CLK : IN std_logic;
		WB_CTI_I : IN std_logic_vector(2 downto 0);
		WB_BTE_I : IN std_logic_vector(1 downto 0);
		WB_ADR_I : IN std_logic_vector(31 downto 0);
		WB_DAT_I : IN std_logic_vector(31 downto 0);
		WB_SEL_I : IN std_logic_vector(3 downto 0);
		WB_WE_I : IN std_logic;
		WB_STB_I : IN std_logic;
		WB_CYC_I : IN std_logic;
		WB_ACK_O : OUT std_logic;
		WB_DAT_O : OUT std_logic_vector(31 downto 0);
		WB_ERR_O : OUT std_logic;
		WB_RTY_O : OUT std_logic;
		RDY : OUT std_logic

   );
end SPI_top_RW;
-----------------------------------------------------------------------
architecture rtl_SPI_top of SPI_top_RW is
    component memoriadp
        port (DataInA : in std_logic_vector(15 downto 0); 
        DataInB : in std_logic_vector(15 downto 0); 
        AddressA : in std_logic_vector(3 downto 0); 
        AddressB : in std_logic_vector(3 downto 0); ClockA: in std_logic; 
        ClockB: in std_logic; ClockEnA: in std_logic; 
        ClockEnB: in std_logic; WrA: in std_logic; WrB: in std_logic; 
        ResetA: in std_logic; ResetB: in std_logic; 
        QA : out std_logic_vector(15 downto 0); 
        QB : out std_logic_vector(15 downto 0)
    );
    end component;
constant clk_rising : std_logic := '1';
signal input_SR : std_logic_vector (15 downto 0);
signal output_SR : std_logic_vector (15 downto 0);
signal u_spi_clk_smp1 : std_logic;  
signal u_spi_clk_smp2 : std_logic; 
signal u_spi_csn_smp1 : std_logic;  
signal u_spi_csn_smp2 : std_logic;  
signal u_spi_csn_smp3 : std_logic; 	
signal u_spi_csn_smp4 : std_logic; 
signal u_spi_csn_smp5 : std_logic; 
signal AddressA 		: std_logic_vector(3 downto 0);
signal WData				: std_logic_vector(15 downto 0);
signal RData				: std_logic_vector(15 downto 0);
signal WE_int : std_logic;  
signal WE : std_logic; 
signal notrst : std_logic; 

signal proceduraread_GO1 : std_logic;
signal proceduraread_GO2 : std_logic;

signal Q				: std_logic_vector(15 downto 0);
signal RdAddress        :std_logic_vector(3 downto 0);

signal WrB : std_logic;

begin


process(rst, WB_CLK)
begin
if rst = '0' then
	WB_ACK_O <= '0';
elsif WB_CLK'event and WB_CLK='1' then
	WB_ACK_O <= WB_STB_I and WB_CYC_I;
end if;
end process;
---------- input sr
process(rst, WB_CLK)
begin
if rst = '0' then
	output_SR <= "0000000000000000";
	input_SR <= "0000000000000000";
	WE_int <= '0';
	AddressA <= "0000";
	RDY <= '0';
	proceduraread_GO1 <= '0';
	proceduraread_GO2 <= '0';
elsif WB_CLK'event and WB_CLK='1' then
	u_spi_clk_smp1 <= u_spi_clk;
	u_spi_clk_smp2 <= u_spi_clk_smp1;
	u_spi_csn_smp1 <= u_spi_csn;
	u_spi_csn_smp2 <= u_spi_csn_smp1;
	u_spi_csn_smp3 <= u_spi_csn_smp2;  
	u_spi_csn_smp4 <= u_spi_csn_smp3;
	u_spi_csn_smp5 <= u_spi_csn_smp4;
	WE_int <= '0';
	WE <= WE_int ;
	
	if u_spi_clk_smp2 = '1' and u_spi_clk_smp1 = '0' and u_spi_csn = '0' then -- discesa (attenzione inverso al cliente), CS attivo basso
		input_SR(15 downto 0) <= input_SR(14 downto 0) & u_spi_d;
	end if;
	
	if u_spi_clk_smp2 = '0' and u_spi_clk_smp1 = '1' and u_spi_csn = '0' then 
		u_spi_q <= output_SR(15); 
	end if;
	
	if u_spi_clk_smp2 = '0' and u_spi_clk_smp1 = '1' and u_spi_csn = '0' then 
		output_SR(15 downto 0) <= output_SR(14 downto 0) & '0';
	end if;
	
	if u_spi_csn_smp3 = '0' and u_spi_csn_smp2 = '1'  then -- salita CS
		if 		input_SR(15 downto 8) = "00000000"  then -- preambolo write 0x00 
			AddressA <= input_SR(3 downto 0) ;
		elsif   input_SR(15 downto 8) = "10000000" then -- preambolo  read 0x80
			AddressA <= input_SR(3 downto 0) ;
			proceduraread_GO1 <= '1';
		elsif	input_SR(15 downto 8) = "00000001" then -- preambolo  write dato alto 0x01
			WData(15 downto 8) <= input_SR(7 downto 0);
		elsif	input_SR(15 downto 8) = "00000010" then -- preambolo  write dato basso 0x02
			WData(7 downto 0) <= input_SR(7 downto 0);
			WE_int <= '1';		
		--elsif	input_SR(15 downto 8) = "10000010" then -- preambolo  read dato basso 0x82
		--	output_SR <= X"81" & RData(7 downto 0);
		end if;
	end if;
	
	if u_spi_csn_smp5 = '0' and u_spi_csn_smp4 = '1'  then -- salita CS ritardata
		if      proceduraread_GO1 = '1' then 
			output_SR <= X"81" & RData(15 downto 8);
			proceduraread_GO1 <= '0' ;
			proceduraread_GO2 <= '1';
		elsif      proceduraread_GO2 = '1' then 
			output_SR <= X"82" & RData(7 downto 0);
			proceduraread_GO2 <= '0';
		end if ;
	end if;
end if;
end process;   
notrst <= not rst;	

    u1 : memoriadp
        port map (
		ClockA => WB_CLK, 
        ClockEnA => '1', 
		ResetA => notrst,
		ClockB => WB_CLK, 
		ClockEnB => '1',
		ResetB => notrst,
		---------------------
		DataInA => WData, 
		AddressA => AddressA, 
		WrA => WE, 
		QA => RData, 
		---------------------
		DataInB => WB_DAT_I(15 downto 0), 
		AddressB => RdAddress,
		WrB => '0', 
		QB =>Q 
        );


WB_DAT_O <= Q & Q;

RdAddress(3 downto 0)  <= WB_ADR_I(4 downto 1);

WrB <= WB_STB_I and WB_CYC_I and WB_WE_I;

end rtl_SPI_top;
-----------------------------------------------------------------------