-- ====================================================================
-- TFT_RIDER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver x.x - date xx gen 2007
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
-----------------------------------------------------------------------
entity tftsurfer_demo is
   port(
      -- system interface
      sys_rst         : in std_logic;
      tft_clk         : in std_logic;
      demo_start      : in std_logic;
		-- test point x debug
		test_pt         : out std_logic;
		-- tft_timing_ctrl interface
		tft_bank        : out std_logic_vector(1 downto 0);
		tft_sect        : out std_logic_vector(2 downto 0);
      -- pixel_addr_data_fifo interface
      fifo_wr_req     : out std_logic;
      fifo_wr_add     : out std_logic_vector(21 downto 0);
      fifo_wr_rgb     : out std_logic_vector(15 downto 0);
      -- spi flash interface
      spi_csn         : out std_logic;
      spi_clk         : out std_logic;
      spi_d           : out std_logic;
      spi_q           : in std_logic
   );
end tftsurfer_demo;
-----------------------------------------------------------------------
architecture rtl_tftsurfer_demo of tftsurfer_demo is

   component micro_emulator
		port(
			-- system interface
			sys_rst         : in std_logic;
			tft_clk         : in std_logic;
			demo_start      : in std_logic;
			-- micro_ctrl_if interface
			demo_sel        : out std_logic; -- OPEN
			tft_surfer_busy : out std_logic; -- TEST_POINT
			-- img_mover_viewer interface
			img_start_move  : out std_logic; -- attivo alto un solo tft_clk
			img_view        : out std_logic; -- attivo alto un solo tft_clk
			img_nr          : out std_logic_vector(3 downto 0);
			img_end_move    : in std_logic
		);
   end component;

   component img_mover_viewer
		port(
			-- system interface
			sys_rst        : in std_logic;
			tft_clk        : in std_logic;
			-- micro_emulator interface
			img_start_move : in std_logic; -- attivo alto un solo tft_clk
			img_view       : in std_logic; -- attivo alto un solo tft_clk
			img_nr         : in std_logic_vector(3 downto 0);
			img_end_move   : out std_logic;
			-- pixel_addr_data_fifo interface
			fifo_wr_req    : out std_logic;
			fifo_wr_add    : out std_logic_vector(21 downto 0);
			fifo_wr_rgb    : out std_logic_vector(15 downto 0);
			-- tft_timing_ctrl interface
			tft_bank       : out std_logic_vector(1 downto 0);
			tft_sect       : out std_logic_vector(2 downto 0);
			-- spi_flash_reader interface
			spi_start_rd   : out std_logic;
			spi_addr       : out std_logic_vector(7 downto 0);
			spi_byte_nr    : out std_logic_vector(20 downto 0);
			spi_byte_en    : in std_logic;
			spi_byte       : in std_logic_vector(7 downto 0);
			spi_end_rd     : in std_logic
		);
   end component;

   component spi_flash_reader
		port(
			-- system interface
			sys_rst      : in std_logic;
			spi_clk_in   : in std_logic;
			-- spi flash interface
			spi_csn      : out std_logic;
			spi_clk      : out std_logic;
			spi_d        : out std_logic;
			spi_q        : in std_logic;
			-- micro_emulator interface
			spi_start_rd : in std_logic;
			spi_addr     : in std_logic_vector(7 downto 0);
			spi_byte_nr  : in std_logic_vector(20 downto 0);
			spi_byte_en  : out std_logic;
			spi_byte     : out std_logic_vector(7 downto 0);
			spi_end_rd   : out std_logic
		);
   end component;

   signal spi_start_rd_sig : std_logic;
   signal spi_addr_sig     : std_logic_vector(7 downto 0);
   signal spi_byte_nr_sig  : std_logic_vector(20 downto 0);
   signal spi_byte_en_sig  : std_logic;
   signal spi_byte_sig     : std_logic_vector(7 downto 0);
   signal spi_end_rd_sig   : std_logic;

   signal img_start_move_sig : std_logic;
   signal img_view_sig       : std_logic;
   signal img_nr_sig         : std_logic_vector(3 downto 0);
   signal img_end_move_sig   : std_logic;

begin

   MICRO_EMULATOR_INST: micro_emulator
		port map(
			-- system interface
			sys_rst         => sys_rst,
			tft_clk         => tft_clk,
			demo_start      => demo_start,
			-- micro_ctrl_if interface
			demo_sel        => open,
			tft_surfer_busy => test_pt,
			-- img_mover_viewer interface
			img_start_move  => img_start_move_sig,
			img_view        => img_view_sig,
			img_nr          => img_nr_sig,
			img_end_move    => img_end_move_sig
		);

   IMG_MOVER_VIEWER_INST: img_mover_viewer
      port map(
			-- system interface
			sys_rst        => sys_rst,
			tft_clk        => tft_clk,
			-- micro_emulator interface
			img_start_move => img_start_move_sig,
			img_view       => img_view_sig,
			img_nr         => img_nr_sig,
			img_end_move   => img_end_move_sig,
			-- pixel_addr_data_fifo interface
			fifo_wr_req    => fifo_wr_req,
			fifo_wr_add    => fifo_wr_add,
			fifo_wr_rgb    => fifo_wr_rgb,
			-- tft_timing_ctrl interface
			tft_bank       => tft_bank,
			tft_sect       => tft_sect,
			-- spi_flash_reader interface
         spi_start_rd   => spi_start_rd_sig,
         spi_addr       => spi_addr_sig,
         spi_byte_nr    => spi_byte_nr_sig,
         spi_byte_en    => spi_byte_en_sig,
         spi_byte       => spi_byte_sig,
         spi_end_rd     => spi_end_rd_sig
		);

   SPI_FLASH_READER_INST: spi_flash_reader
      port map(
			-- system interface
			sys_rst      => sys_rst,
			spi_clk_in   => tft_clk,
			-- spi flash interface
			spi_csn      => spi_csn,
			spi_clk      => spi_clk,
			spi_d        => spi_d,
			spi_q        => spi_q,
			-- micro_emulator interface
         spi_start_rd => spi_start_rd_sig,
         spi_addr     => spi_addr_sig,
         spi_byte_nr  => spi_byte_nr_sig,
         spi_byte_en  => spi_byte_en_sig,
         spi_byte     => spi_byte_sig,
         spi_end_rd   => spi_end_rd_sig
		);

end rtl_tftsurfer_demo;
-----------------------------------------------------------------------